(PackageInfo named: 'Sudoku') preamble: '"below, add code to be run before the loading of this package"
(Installer mc http: ''http://www.hpi.uni-potsdam.de/hirschfeld/squeaksource/'')
   project: ''MetacelloRepository'';
   install: ''ConfigurationOfWidgets''.
(Smalltalk at: #ConfigurationOfWidgets) load.
'!
SystemOrganization addCategory: #'Sudoku-Strategies'!
SystemOrganization addCategory: #'Sudoku-Calculation'!
SystemOrganization addCategory: #'Sudoku-GUI'!
SystemOrganization addCategory: #'Sudoku-Core'!

RectangleMorph subclass: #SudokuCell
	instanceVariableNames: 'sudokuModel xCoord yCoord'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-GUI'!

SudokuCell subclass: #SudokuCandidateCell
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-GUI'!

!SudokuCandidateCell class methodsFor: 'instance creation' stamp: 'MR 2/1/2013 12:02:18.02'!
new
	"This method creates a Sudoku cell that shows all candidates from 1 to 9."
	| candidateCell |
	candidateCell := SudokuSingleNumberCell new initializeCellSize: 25 textSize: 18 
													updateColor: Color black 
													normalColor: Color black
													highlightColor: Color green
													removeColor: (Color r: 1 g:0 b:0).
	^ self newFrom: ((SudokuBuilder createBoxFromMorph: candidateCell) borderWidth: 1)! !

!SudokuCandidateCell methodsFor: 'visual properties' stamp: 'MG 12/15/2012 16:31'!
highlightCandidates: aCollection
"aCollection contains all candidates that should be highlighted."
	aCollection do: 
		[:candidate | ((self submorphs at: candidate-1 \\ 3 + 1) 
								submorphs at: candidate-1 / 3 + 1) 
								updateContent: candidate ofType: #highlight].! !

!SudokuCandidateCell methodsFor: 'visual properties' stamp: 'MG 12/15/2012 16:44'!
markCandidatesToRemove: aCollection
	aCollection do: 
		[:candidate | ((self submorphs at: candidate-1 \\ 3 + 1) 
								submorphs at: candidate-1 / 3 + 1) 
								updateContent: candidate ofType: #removeCandidates].! !

!SudokuCandidateCell methodsFor: 'converting' stamp: 'mg 6/19/2014 11:24'!
transformCandidateToSingleNumber: aNumber
	| singleNumber |
	singleNumber := SudokuSolvedCell new.
	aNumber ~= 0 ifTrue: [singleNumber submorphs first updateColor: Color black].
	singleNumber updateContent: aNumber ofType: #number.
	
	owner := self owner replaceSubmorph: self by: singleNumber.
	self delete.! !

!SudokuCandidateCell methodsFor: 'updating' stamp: 'MR 12/9/2012 23:46'!
updateCandidates: aCollection
	self submorphs do:
		[:threeCandidates | threeCandidates submorphs do:
			[:candidate | candidate updateContent: ' ' ofType: #number]].
	
	aCollection do: 
		[:candidate | ((self submorphs at: candidate-1 \\ 3 + 1) 
								submorphs at: candidate-1 / 3 + 1) 
								updateContent: candidate ofType: #number].! !

!SudokuCandidateCell methodsFor: 'updating' stamp: 'mg 6/18/2014 12:25'!
updateContent: content ofType: aSymbol
	(aSymbol = #number and: [(content ~= '')])
		ifTrue: [self transformCandidateToSingleNumber: content].
	(aSymbol = #candidates)
		ifTrue: [self updateCandidates: content].
	(aSymbol = #highlight)
		ifTrue: [self highlightCandidates: content].
	(aSymbol = #removeCandidates)
		ifTrue: [self markCandidatesToRemove: content].
	self changed.! !

!SudokuCell methodsFor: 'updating' stamp: 'mg 6/23/2014 13:55'!
deleteNumberInSudokuModel

	self sudokuModel disableEqualityConstraintAt: self xCoord at: self yCoord.! !

!SudokuCell methodsFor: 'initialization' stamp: 'mg 6/17/2014 17:58'!
initialize

	super initialize.
	xCoord := 0.
	yCoord := 0.! !

!SudokuCell methodsFor: 'event handling' stamp: 'mg 6/30/2014 13:28'!
moveFocusBy: aPoint event: anEvent
	"This method is called in interactive solving mode on a mouse enter event. It moves the keyboard focus to the current cell."
	| newFocusedCell textMorph |
	newFocusedCell := self sudokuModel sudokuGui cellAtX: self xCoord + aPoint x Y: self yCoord + aPoint y.
	newFocusedCell ifNil: [^ self].
	textMorph := newFocusedCell submorphs first.
	anEvent hand newKeyboardFocus: textMorph.
	textMorph
		handleInteraction: [textMorph editor mouseDown: anEvent]
		fromEvent: anEvent.! !

!SudokuCell methodsFor: 'accessing' stamp: 'mg 6/17/2014 18:07'!
sudokuModel

	^ sudokuModel! !

!SudokuCell methodsFor: 'accessing' stamp: 'mg 6/17/2014 18:07'!
sudokuModel: anObject

	sudokuModel := anObject! !

!SudokuCell methodsFor: 'updating' stamp: 'mg 6/19/2014 13:40'!
suggestNewValueToSudokuModel: aValue

	self sudokuModel createConstraintAt: self xCoord at: self yCoord with: aValue.! !

!SudokuCell methodsFor: 'updating' stamp: 'MG 12/7/2012 15:17'!
updateContent: content ofType: aSymbol
	self subclassResponsibility.! !

!SudokuCell methodsFor: 'accessing' stamp: 'mg 6/17/2014 17:58'!
xCoord

	^ xCoord! !

!SudokuCell methodsFor: 'accessing' stamp: 'mg 6/17/2014 17:58'!
xCoord: anObject

	xCoord := anObject! !

!SudokuCell methodsFor: 'accessing' stamp: 'mg 6/17/2014 17:58'!
yCoord

	^ yCoord! !

!SudokuCell methodsFor: 'accessing' stamp: 'mg 6/17/2014 17:58'!
yCoord: anObject

	yCoord := anObject! !

SudokuCell subclass: #SudokuGivenNumberCell
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-GUI'!

!SudokuGivenNumberCell methodsFor: 'initialization' stamp: 'mg 6/25/2014 13:50'!
initialize

	super initialize.
	self initializeCellSize: 75 textSize: 36. 
	self borderWidth: 0.! !

!SudokuGivenNumberCell methodsFor: 'initialization' stamp: 'mg 6/25/2014 11:31'!
initializeCellSize: cellSize textSize: textSize
	| textMorph |
	super initialize.
	self
		layoutPolicy: TableLayout new;
		color: Color white;
		borderWidth: 0;
		width: cellSize;
		height: cellSize;
		disableDragNDrop;
		listCentering: #center;
		wrapCentering: #center.
		
	textMorph := TextMorph new
		borderWidth: 0; 
		width: cellSize/2;
		height: cellSize/2;
		beAllFont: (((TextStyle named: 'BitstreamVeraSans') fontOfSize: textSize) emphasized: 1);
		disableDragNDrop;
		contents: ' ';
		centered.

	self addMorph: textMorph.! !

!SudokuGivenNumberCell methodsFor: 'e-toy support' stamp: 'mg 6/25/2014 13:47'!
setNumericValue: aValue

	self submorphs first setNumericValue: aValue.! !

SudokuCell subclass: #SudokuSingleNumberCell
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-GUI'!

!SudokuSingleNumberCell methodsFor: 'initialization' stamp: 'mg 6/19/2014 11:20'!
initialize
	"This method initializes all 81 Sudoku cells."
	super initialize.
	self initializeCellSize: 77 
		 textSize: 36 
		 updateColor: Color blue 
		 normalColor: Color black 
		 highlightColor: Color green 
		 removeColor: Color blue. 
	self 
		borderWidth: 1.! !

!SudokuSingleNumberCell methodsFor: 'initialization' stamp: 'MR 2/1/2013 12:07:18.91'!
initializeCellSize: cellSize textSize: textSize updateColor: aColor1 normalColor: aColor2 highlightColor: aColor3 removeColor: aColor4
	| textMorph |
	super initialize.
	self
		layoutPolicy: TableLayout new;
		color: Color white;
		borderWidth: 0;
		width: cellSize;
		height: cellSize;
		disableDragNDrop;
		listCentering: #center;
		wrapCentering: #center.
		
	textMorph := SudokuTextMorph new
		borderWidth: 0; 
		width: cellSize/2;
		height: cellSize/2;
		beAllFont: (((TextStyle named: 'BitstreamVeraSans') fontOfSize: textSize) emphasized: 1);
		disableDragNDrop;
		setNumericValue: 0;
		normalColor: aColor2;
		updateColor: aColor1;
		highlightColor: aColor3;
		removeCandidatesColor: aColor4;
		centered.

	self addMorph: textMorph.! !

!SudokuSingleNumberCell methodsFor: 'updating' stamp: 'mg 1/10/2013 16:52:40.749'!
updateContent: candidate ofType: aSymbol
	(aSymbol = #number)
		ifTrue: [self submorphs first updateTextWith: candidate; setToWhiteBackground].
	(aSymbol = #highlight)
		ifTrue: [self submorphs first setToNormalColor; setToHighlightColor].
	(aSymbol = #removeCandidates)
		ifTrue: [self submorphs first setToRemoveCandidatesColor].
	(aSymbol = #candidates)
		ifTrue: [self submorphs first setToWhiteBackground].  
	self changed.! !

SudokuSingleNumberCell subclass: #SudokuSolvedCell
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-GUI'!

!SudokuSolvedCell methodsFor: 'updating' stamp: 'mg 2/2/2013 20:52:43.923'!
transformSolvedCellToCandidate: aCollection
	| cell |
	cell := SudokuCandidateCell new.
	cell updateCandidates: aCollection.
	
	owner := self owner replaceSubmorph: self by: cell.
	self delete.! !

!SudokuSolvedCell methodsFor: 'updating' stamp: 'mg 6/18/2014 12:40'!
updateContent: solvedNumber ofType: aSymbol
	(aSymbol = #number)
		ifTrue: [(solvedNumber = 0)
					ifTrue: [self submorphs first updateTextWith: '']
					ifFalse: [self submorphs first updateTextWith: solvedNumber]].
	(aSymbol = #highlight)
		ifTrue: [self submorphs first setToNormalColor.
				(solvedNumber notEmpty) 
					ifTrue: [self submorphs first setToHighlightColor]].
	(aSymbol = #removeCandidates)
		ifTrue: ["do nothing because here are no candidates"].
	(aSymbol = #candidates)
		ifTrue: ["In historyMode, it is possible that solvedNumber is a collection of multiple candidiates. In that case, transform the solved cell back into a candidate cell."
			(solvedNumber isCollection and: [solvedNumber size > 1]) 
				ifTrue: [self transformSolvedCellToCandidate: solvedNumber].  
			self submorphs first setToWhiteBackground].  
	self changed.! !

RectangleMorph subclass: #SudokuStrategyDisplay
	instanceVariableNames: 'textLines'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-GUI'!

!SudokuStrategyDisplay methodsFor: 'initialization' stamp: 'MR 1/10/2013 16:55'!
createLine: aString
	| line |
	
	line := TextMorph new.
	line
		beAllFont: (((TextStyle named: 'BitstreamVeraSans') fontOfSize: 24) emphasized: 1);
		contents: aString;
		backgroundColor: Color white;
		autoFit: False;
		hResizing: #spaceFill;
		disableDragNDrop.
	
	textLines add: line.! !

!SudokuStrategyDisplay methodsFor: 'visual properties' stamp: 'MR 2/2/2013 16:15:44.345'!
highlightLine: anInteger
	| line |

	self submorphs do:
		[:morph | morph 
					backgroundColor: Color white; 
					textColor: Color black].
	line := self submorphs at: anInteger.
	line 
		backgroundColor: Color green; 
		textColor: Color white.! !

!SudokuStrategyDisplay methodsFor: 'initialization' stamp: 'MR 1/12/2013 15:55:26.043'!
initialize

	super initialize.
	textLines := OrderedCollection new.
	self 
		layoutPolicy: TableLayout new;
		listDirection: #topToBottom;
		hResizing: #rigid;
		vResizing: #spaceFill;
		width: 300;
		wrapCentering: #topLeft;
		layoutInset: 10;
		borderWidth: 3;
		borderColor: Color black;
		color: Color white;
		disableDragNDrop.
		
	
		! !

!SudokuStrategyDisplay methodsFor: 'initialization' stamp: 'MR 1/10/2013 16:59'!
initializeStrategyTexts: anOrderedCollection

	anOrderedCollection do:
		[:strategy| self createLine: (strategy strategyName).].
	
	self addAllMorphs: textLines! !

!SudokuStrategyDisplay methodsFor: 'visual properties' stamp: 'MR 2/2/2013 16:14:00.705'!
stopHighlighting

	self submorphs do:
		[:morph | morph 
						backgroundColor: Color white; 
						textColor: Color black.].! !

IconicButton subclass: #SudokuPlayButton
	instanceVariableNames: 'state playForm pauseForm sudokuSolver'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-GUI'!

!SudokuPlayButton methodsFor: 'initialization' stamp: 'MR 1/12/2013 19:59:01.664'!
initialize

	super initialize.
	state := false.
	self stopStepping.! !

!SudokuPlayButton methodsFor: 'accessing' stamp: 'MR 1/12/2013 17:38:44.958'!
pauseForm
	"Answer the value of pauseForm"

	^ pauseForm! !

!SudokuPlayButton methodsFor: 'accessing' stamp: 'MR 1/12/2013 17:39:14.606'!
pauseForm: aForm
	"Set the value of pauseForm"

	pauseForm := aForm! !

!SudokuPlayButton methodsFor: 'accessing' stamp: 'MR 1/12/2013 17:38:44.899'!
playForm
	"Answer the value of playForm"

	^ playForm! !

!SudokuPlayButton methodsFor: 'accessing' stamp: 'MR 1/12/2013 17:42:49.28'!
playForm: aForm
	"Set the value of playForm"

	playForm := aForm.
	self labelGraphic: playForm.! !

!SudokuPlayButton methodsFor: 'stepping and presenter' stamp: 'MR 1/12/2013 20:06:51.194'!
playOrPause

	state 
		ifTrue: [self stopStepping.
				self labelGraphic: playForm]
		ifFalse: [self startStepping.
				self labelGraphic: pauseForm].
	state := state not.! !

!SudokuPlayButton methodsFor: 'accessing' stamp: 'MR 1/12/2013 17:38:44.827'!
state
	"Answer the value of state"

	^ state! !

!SudokuPlayButton methodsFor: 'accessing' stamp: 'MR 1/12/2013 17:39:41.788'!
state: aBoolean
	"Set the value of state"

	state := aBoolean! !

!SudokuPlayButton methodsFor: 'stepping and presenter' stamp: 'MR 2/2/2013 16:08:40.152'!
step

	super step.
	state ifTrue: [sudokuSolver takeStep
					ifFalse: [self playOrPause]].! !

!SudokuPlayButton methodsFor: 'stepping and presenter' stamp: 'mg 1/17/2013 15:35:40.419'!
stepTime
	^ 200! !

!SudokuPlayButton methodsFor: 'accessing' stamp: 'MR 1/12/2013 17:38:45.008'!
sudokuSolver
	"Answer the value of sudokuSolver"

	^ sudokuSolver! !

!SudokuPlayButton methodsFor: 'accessing' stamp: 'MR 1/12/2013 17:39:52.906'!
sudokuSolver: aSudokuSolver
	"Set the value of sudokuSolver"

	sudokuSolver := aSudokuSolver! !

Object subclass: #BacktrackingStrategy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Strategies'!

!BacktrackingStrategy methodsFor: 'as yet unclassified' stamp: 'mg 2/3/2013 13:48:31.209'!
solveFromCandidates: aMatrix
"The method returns false if the given matrix is not a valid Sudoku grid"
	| inputMatrix |
	(SudokuStrategyRulesChecker finished: aMatrix) ifTrue: [^true].   
	(SudokuStrategyRulesChecker isValid: aMatrix) ifFalse: [^false].   
	inputMatrix := SudokuGrid newFrom: aMatrix. 
	aMatrix withIndicesDo: [:candidates :x :y |
		(candidates size > 1 and: [candidates size < 5])
			ifTrue: ["test all cases"
				candidates do: [:testCandidate | | newGrid attempt rules |
					newGrid := SudokuGrid newFrom: aMatrix.
					(newGrid at: x at: y) removeAll; add: testCandidate.
					rules := SudokuStrategyRulesChecker new. 
					rules solveFromCandidates: newGrid.
					newGrid := rules candidates.
					attempt := BacktrackingStrategy new solveFromCandidates: newGrid.
					attempt 
						ifFalse: [(inputMatrix at: x at: y) remove: testCandidate]
						ifTrue: [^true]].
				^BacktrackingStrategy new solveFromCandidates: inputMatrix]].
	^ true! !

Object subclass: #SudokuBuilder
	instanceVariableNames: 'sudokuGUI sudokuSolver strategies'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-GUI'!

!SudokuBuilder class methodsFor: 'building' stamp: 'MR 12/15/2012 17:12'!
createBoxFromMorph: aMorph
	"This method creates a 3x3 box. The submorphs of the box are copies of aMorph."
	| column box |
	
	column := self createTableOfThreeCellsWithDirection: #topToBottom fromMorph: aMorph.
	box := self createTableOfThreeCellsWithDirection: #leftToRight fromMorph: column.
	^ box! !

!SudokuBuilder class methodsFor: 'building' stamp: 'MR 12/15/2012 17:28'!
createTableOfThreeCellsWithDirection: aSymbol fromMorph: aMorph
	"This method creates three cells in a row (aSymbol = #leftToRight) or column (aSymbol = #topToBottom)."
	| table |
	
	table := RectangleMorph new
		layoutPolicy: TableLayout new;
		listDirection: aSymbol;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap;
		borderWidth: 0;
		layoutInset: 0;
		color: Color black;
		disableDragNDrop.
	3 timesRepeat: [table addMorph: aMorph copy].
	^ table! !

!SudokuBuilder methodsFor: 'building' stamp: 'mg 2/4/2013 11:28:07.569'!
buildSudoku
	self buildSudoku: 'sudoku.txt'.! !

!SudokuBuilder methodsFor: 'building' stamp: 'mg 2/4/2013 11:27:16.778'!
buildSudoku: fileName
	self 
		buildSudokuSolver;
		buildSudokuGUI;
		buildSudokuSolverStates.
	sudokuSolver 
		fileName: fileName;  
		takeStep; takeStep.
	sudokuGUI openInWorld.! !

!SudokuBuilder methodsFor: 'building' stamp: 'MR 2/2/2013 19:16:15.283'!
buildSudokuGUI
	| strategyDisplay historySlider |
	sudokuGUI := SudokuGUI new.
	sudokuGUI createButtons: sudokuSolver.
	historySlider := sudokuGUI createSlider: sudokuSolver.
	strategyDisplay := sudokuGUI createStrategyDisplay: strategies.
	sudokuGUI connectSubmorphs.
	sudokuSolver strategyDisplay: strategyDisplay.
	sudokuSolver historySlider: historySlider.! !

!SudokuBuilder methodsFor: 'building' stamp: 'mg 2/3/2013 01:12:03.354'!
buildSudokuSolver
	sudokuSolver := SudokuSolver new. 
	strategies := OrderedCollection 
					with: SudokuStrategyRulesChecker new 
					with: SudokuStrategyHiddenSingles new 
					with: SudokuStrategyNakedPairs new
					with: SudokuStrategyHiddenPairs new
					with: SudokuStrategyPointingPairs new
					with: SudokuStrategyBoxLineReduction new.
	strategies add: SudokuStrategyYWing new.
	sudokuSolver strategies: strategies.! !

!SudokuBuilder methodsFor: 'building' stamp: 'MG 12/15/2012 16:37'!
buildSudokuSolverStates
	| allStates |
	allStates := OrderedCollection new.
	allStates 
		add: SudokuStateHighlightCandidates new;
		add: SudokuStateCandidatesToRemove new;
		add: SudokuStateCandidates new;
		add: SudokuStateNumbers new.
		
	allStates do:
		[:state | state sudokuGUI: sudokuGUI; sudokuSolver: sudokuSolver].
	
	sudokuSolver 
		allStates: allStates;
		currentState: 3.
! !

Object subclass: #SudokuSolvedCellsChecker
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Calculation'!

!SudokuSolvedCellsChecker methodsFor: 'solving' stamp: 'MG 11/24/2012 15:40'!
findSolvedCellsOf: aMatrix
"This method finds all lists of candidates that contain only one number."
	| numbers |
	numbers := Matrix new: aMatrix rowCount.
	1 to: aMatrix rowCount do:
		[:i | 1 to: aMatrix columnCount do: 
			[:j | ((aMatrix at: i at: j) size == 1)
				ifTrue: [numbers at: i at: j put: ((aMatrix at: i at: j) first)] ]].
	^ numbers! !

Object subclass: #SudokuSolver
	instanceVariableNames: 'strategies currentStrategy currentState allStates strategyDisplay candidatesHistory historyPosition historySlider fileName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Calculation'!

!SudokuSolver methodsFor: 'accessing' stamp: 'MG 12/14/2012 14:32'!
allStates: anObject
	"Set the value of allStates"

	allStates := anObject! !

!SudokuSolver methodsFor: 'calculation' stamp: 'MR 2/2/2013 15:59:29.137'!
applyStrategy
	strategies 
		detect: [:strategy | strategy solveFromCandidates: self candidatesFromStrategy] 
		ifFound: [:solvingStrategy | currentStrategy := solvingStrategy.
									Transcript show: currentStrategy description.
									strategyDisplay highlightLine: (strategies indexOf: currentStrategy).
									^true]
		ifNone: [Transcript show: 'No strategy found'.
				strategyDisplay stopHighlighting.
				^false]! !

!SudokuSolver methodsFor: 'calculation' stamp: 'MR 2/2/2013 17:20:34.027'!
applyStrategyForCandidates: anOrderedCollection

	strategies 
		detect: [:strategy | strategy solveFromCandidates: anOrderedCollection] 
		ifFound: [:solvingStrategy | currentStrategy := solvingStrategy.
									Transcript show: currentStrategy description.
									strategyDisplay highlightLine: (strategies indexOf: currentStrategy).
									^true]
		ifNone: [Transcript show: 'No strategy found'.
				strategyDisplay stopHighlighting.
				^false]! !

!SudokuSolver methodsFor: 'calculation' stamp: 'MR 2/2/2013 19:55:04.593'!
calculationInHistoryMode

	self applyStrategyForCandidates: (candidatesHistory at: historyPosition).
	historyPosition := historyPosition + 1.
	historySlider relativeValue: historyPosition! !

!SudokuSolver methodsFor: 'calculation' stamp: 'MR 2/2/2013 20:01:39.001'!
calculationInNormalMode

	self applyStrategy
		ifFalse: [self inform: 'No Strategy found!!'.
				^false]. 
		self storeCandidates.
		historySlider 
			value: 1;
			maxRelativeValue: candidatesHistory size.
	^true! !

!SudokuSolver methodsFor: 'accessing' stamp: 'MR 2/2/2013 18:36:51.906'!
candidates
	"Answer the value of candidates"

	candidatesHistory 
		ifEmpty: [candidatesHistory add: self initialCandidates].
	historyPosition == 0
		ifFalse: [^ candidatesHistory at: historyPosition].
	^ candidatesHistory last! !

!SudokuSolver methodsFor: 'accessing' stamp: 'MR 2/2/2013 16:48:53.124'!
candidatesFromStrategy
	
	currentStrategy 
		ifNotNil: [^ currentStrategy candidates]
		ifNil: [^ self candidates]! !

!SudokuSolver methodsFor: 'accessing' stamp: 'MR 12/22/2012 18:26'!
currentState
	"Answer the value of the current state as a state (not an index)"

	^ allStates at: currentState! !

!SudokuSolver methodsFor: 'accessing' stamp: 'MR 12/22/2012 18:26'!
currentState: anInteger
	"Set the index of currentState"

	currentState := anInteger ! !

!SudokuSolver methodsFor: 'as yet unclassified' stamp: 'mg 2/4/2013 11:26:25.311'!
fileName: aString
"Sets the file name of the Sudoku."
	fileName := aString.! !

!SudokuSolver methodsFor: 'accessing' stamp: 'MG 12/15/2012 15:14'!
highlightCandidates
"This method returns a matrix that contains collections with all candidates that should be highlighted."
	^ currentStrategy highlightCandidates! !

!SudokuSolver methodsFor: 'accessing' stamp: 'MR 2/2/2013 19:02:10.632'!
historySlider: aSudokuSlider
	"Set the historySlider"

	historySlider := aSudokuSlider  ! !

!SudokuSolver methodsFor: 'calculation' stamp: 'MR 2/2/2013 20:00:05.41'!
inHistoryMode 
	"test whether historyPosition is in bounds so that we can use candidatesHistory"
	(historyPosition == 0)
		ifTrue: [^false].
	(historyPosition >= candidatesHistory size)
		ifTrue: [historyPosition := 0. "stop history mode"
				^false].
	^true! !

!SudokuSolver methodsFor: 'initialize-release' stamp: 'mg 2/4/2013 11:29:28.701'!
initialCandidates
	| allCandidates file numberList numbers candidates |

	numberList := OrderedCollection new.
	file := MultiByteFileStream fileNamed: fileName.
	file upToEnd asString substrings
		collect: [:row | row do: [:digit | numberList add: digit digitValue]].

	numbers := Matrix new: 9 tabulate: [:i :j | numberList at: (j \\ 10 + (i-1*9))].
	file close.
	
	allCandidates := OrderedCollection new.
	1 to: 9 do:
		[:i | allCandidates add: i].
	
	candidates := Matrix new: 9.
	1 to: 9 do:
		[:i | 1 to: 9 do:
			[:j | ((numbers at: i at: j) = 0)
					ifTrue: [candidates at: i at: j put: allCandidates copy]
					ifFalse: [candidates at: i at: j put: (OrderedCollection with: (numbers at: i at: j))]]].
	^ candidates! !

!SudokuSolver methodsFor: 'initialization' stamp: 'MR 2/2/2013 18:37:34.871'!
initialize

	super initialize.
	
	candidatesHistory := OrderedCollection new.
	historyPosition := 0. ! !

!SudokuSolver methodsFor: 'accessing' stamp: 'MG 12/15/2012 15:48'!
numbers
	^ SudokuSolvedCellsChecker new findSolvedCellsOf: self candidates.! !

!SudokuSolver methodsFor: 'accessing' stamp: 'MG 12/15/2012 16:17'!
removeCandidates
"This method returns a matrix that contains collections with all candidates that should be highlighted."
	^ currentStrategy removeCandidates! !

!SudokuSolver methodsFor: 'accessing' stamp: 'MR 2/2/2013 17:39:32.703'!
sizeOfHistory

	^ candidatesHistory size! !

!SudokuSolver methodsFor: 'calculation' stamp: 'MR 2/2/2013 16:59:07.929'!
storeCandidates 
	
	candidatesHistory add: self candidatesFromStrategy.! !

!SudokuSolver methodsFor: 'accessing' stamp: 'MR 1/10/2013 15:02'!
strategies: anOrderedCollection
	"Set the strategies"

	strategies := anOrderedCollection ! !

!SudokuSolver methodsFor: 'accessing' stamp: 'mg 1/10/2013 21:15:48.455'!
strategyDisplay: aStrategyDisplay
	"Set the strageyDisplay"

	strategyDisplay := aStrategyDisplay ! !

!SudokuSolver methodsFor: 'calculation' stamp: 'MR 2/2/2013 20:02:34.133'!
takeStep

	(currentState = 1)
		ifTrue: [self inHistoryMode
					ifTrue: [self calculationInHistoryMode]
					ifFalse: [self calculationInNormalMode 
								ifFalse: [^false]]].
	self updateState.
	^true! !

!SudokuSolver methodsFor: 'calculation' stamp: 'MR 2/2/2013 18:35:07.622'!
takeStepFromHistoryAt: anInteger

	currentState := 3.
	historyPosition := anInteger.
	self 
		updateState; "set candidates"
		updateState. "set numbers"
	^true! !

!SudokuSolver methodsFor: 'calculation' stamp: 'MR 2/2/2013 16:19:14.346'!
updateState

	self currentState updateSudokuGUI.
	currentState := currentState \\ allStates size + 1. 
	self currentState setButtonName.! !

Object subclass: #SudokuSolverState
	instanceVariableNames: 'emptyCell sudokuSolver sudokuGUI'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Calculation'!

!SudokuSolverState methodsFor: 'naming' stamp: 'MR 12/22/2012 17:18'!
description 
	"explanation of what will be done in this state"
	self subclassResponsibility! !

!SudokuSolverState methodsFor: 'initialize-release' stamp: 'MG 12/14/2012 15:23'!
initialize
	super initialize.
	emptyCell := nil.! !

!SudokuSolverState methodsFor: 'updating' stamp: 'MR 1/10/2013 15:29'!
setButtonName 

	sudokuGUI setButtonName: self stateName.! !

!SudokuSolverState methodsFor: 'updating' stamp: 'MR 2/2/2013 18:32:43.703'!
setSudokuGUI
	"some states do not support this method"! !

!SudokuSolverState methodsFor: 'naming' stamp: 'MR 1/10/2013 15:28'!
stateName
	"name of the state which should be shown to the user"
	self subclassResponsibility ! !

!SudokuSolverState methodsFor: 'accessing' stamp: 'MG 12/14/2012 14:34'!
sudokuGUI: anObject
	"Set the value of sudokuGUI"

	sudokuGUI := anObject! !

!SudokuSolverState methodsFor: 'accessing' stamp: 'MG 12/14/2012 15:14'!
sudokuSolver: anObject
	"Set the value of sudokuSolver"

	sudokuSolver := anObject! !

!SudokuSolverState methodsFor: 'updating' stamp: 'MG 12/14/2012 14:38'!
updateSudokuGUI
	self subclassResponsibility.! !

SudokuSolverState subclass: #SudokuStateCandidates
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Calculation'!

!SudokuStateCandidates methodsFor: 'naming' stamp: 'MR 12/22/2012 17:42'!
description 

	^ 'remove candidates which were found as being removable'! !

!SudokuStateCandidates methodsFor: 'updating' stamp: 'MR 2/2/2013 18:32:12.533'!
setSudokuGUI
	sudokuGUI setCells: sudokuSolver candidates ofType: #candidates.! !

!SudokuStateCandidates methodsFor: 'naming' stamp: 'MR 1/10/2013 15:29'!
stateName

	^ 'Delete Removable Candidates'! !

!SudokuStateCandidates methodsFor: 'updating' stamp: 'MR 2/2/2013 16:50:07.444'!
updateSudokuGUI
	sudokuGUI updateCells: sudokuSolver candidates ofType: #candidates.! !

SudokuSolverState subclass: #SudokuStateCandidatesToRemove
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Calculation'!

!SudokuStateCandidatesToRemove methodsFor: 'naming' stamp: 'MR 12/22/2012 17:42'!
description 

	^ 'show candidates which can be removed in the next step'! !

!SudokuStateCandidatesToRemove methodsFor: 'naming' stamp: 'MR 1/10/2013 15:29'!
stateName

	^ 'Show Removable Candidates'! !

!SudokuStateCandidatesToRemove methodsFor: 'updating' stamp: 'MG 12/15/2012 16:48'!
updateSudokuGUI
	| removeCandidates |
	removeCandidates := sudokuSolver removeCandidates.
	sudokuGUI updateCells: removeCandidates ofType: #removeCandidates.! !

SudokuSolverState subclass: #SudokuStateHighlightCandidates
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Calculation'!

!SudokuStateHighlightCandidates methodsFor: 'naming' stamp: 'MR 12/22/2012 17:43'!
description 

	^ 'highlight candidates which lead to removement of other candidates'! !

!SudokuStateHighlightCandidates methodsFor: 'naming' stamp: 'MR 1/10/2013 15:29'!
stateName

	^ 'Highlight Important Candidates'! !

!SudokuStateHighlightCandidates methodsFor: 'updating' stamp: 'MG 12/15/2012 15:12'!
updateSudokuGUI
	| highlightCandidates |
	highlightCandidates := sudokuSolver highlightCandidates.
	sudokuGUI updateCells: highlightCandidates ofType: #highlight.! !

SudokuSolverState subclass: #SudokuStateNumbers
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Calculation'!

!SudokuStateNumbers methodsFor: 'naming' stamp: 'MR 12/22/2012 17:43'!
description 

	^ 'transform fields with just one candidate to a number'! !

!SudokuStateNumbers methodsFor: 'updating' stamp: 'MR 2/2/2013 18:34:33.366'!
setSudokuGUI
	| numbers numbersAsString |
	
	numbers := sudokuSolver numbers.
	numbersAsString := Matrix new: 9.
	1 to: 9 do:
		[:i | 1 to: 9 do:
			[:j | ((numbers at: i at: j) == emptyCell)
					ifTrue: [numbersAsString at: i at: j put: '']
					ifFalse: [numbersAsString at: i at: j put: (numbers at: i at: j) asString] ]].
	
	sudokuGUI setCells: numbersAsString ofType: #number.! !

!SudokuStateNumbers methodsFor: 'naming' stamp: 'MR 1/10/2013 15:29'!
stateName

	^ 'Show Solved Cells'! !

!SudokuStateNumbers methodsFor: 'updating' stamp: 'MG 12/15/2012 15:11'!
updateSudokuGUI
	| numbers numbersAsString |
	
	numbers := sudokuSolver numbers.
	numbersAsString := Matrix new: 9.
	1 to: 9 do:
		[:i | 1 to: 9 do:
			[:j | ((numbers at: i at: j) == emptyCell)
					ifTrue: [numbersAsString at: i at: j put: '']
					ifFalse: [numbersAsString at: i at: j put: (numbers at: i at: j) asString] ]].
	
	sudokuGUI updateCells: numbersAsString ofType: #number.! !

Object subclass: #SudokuStrategy
	instanceVariableNames: 'solvedCells inputMatrix candidates highlightCandidates removeCandidates'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Strategies'!

!SudokuStrategy methodsFor: 'accessing' stamp: 'MR 12/15/2012 15:04'!
candidates
	"Answer the value of candidates"

	^ candidates! !

!SudokuStrategy methodsFor: 'accessing' stamp: 'MR 12/15/2012 15:04'!
candidates: anObject
	"Set the value of candidates"

	candidates := anObject! !

!SudokuStrategy methodsFor: 'solving' stamp: 'mg 12/19/2012 13:42:30.046'!
checkStrategyForBoxes
	self subclassResponsibility.! !

!SudokuStrategy methodsFor: 'solving' stamp: 'mg 12/19/2012 13:42:14.894'!
checkStrategyForColumns
	self subclassResponsibility.! !

!SudokuStrategy methodsFor: 'solving' stamp: 'mg 12/19/2012 13:42:01.926'!
checkStrategyForRows
	self subclassResponsibility.! !

!SudokuStrategy methodsFor: 'naming' stamp: 'mg 12/19/2012 14:48:17.668'!
description
	self subclassResponsibility.! !

!SudokuStrategy methodsFor: 'solving' stamp: 'mg 2/2/2013 15:50:55.337'!
findHighlightCandidates! !

!SudokuStrategy methodsFor: 'solving' stamp: 'mg 1/10/2013 22:29:41.24'!
findRemoveCandidates

	removeCandidates := SudokuGrid newFrom: inputMatrix.
	removeCandidates indicesDo:
		[:i :j | (removeCandidates at: i at: j) removeAll: (candidates at: i at:j)]! !

!SudokuStrategy methodsFor: 'accessing' stamp: 'mg 12/19/2012 13:46:13.439'!
getBoxCellsAt: i at: j in: aMatrix
	| cells |
	cells := OrderedCollection new.
	0 to: 2 do:
		[:x | 0 to: 2 do:
			[:y | cells add: (aMatrix at: i+x at: j+y)]].
	^ cells! !

!SudokuStrategy methodsFor: 'accessing' stamp: 'MR 12/15/2012 15:54'!
highlightCandidates
	"Answer the value of highlightCandidates"

	^ highlightCandidates! !

!SudokuStrategy methodsFor: 'accessing' stamp: 'MR 12/15/2012 15:55'!
highlightCandidates: anObject
	"Set the value of highlightCandidates"

	highlightCandidates := anObject! !

!SudokuStrategy methodsFor: 'as yet unclassified' stamp: 'mg 2/1/2013 14:34:32.987'!
indecesPerNumberIn: aCollection
"This method returns a dictionary with all possible numbers as keys. The values are lists of all indices in aCollection, where the number is included. The given collection can be a row of the candidates matrix."
	| dict |
	dict := Dictionary new.
	1 to: aCollection size do: [:number |
		dict at: number put: OrderedCollection new].
	aCollection withIndexDo: [:candidateList :index |
		candidateList do: [:number |
			(dict at: number) add: index]].
	^ dict! !

!SudokuStrategy methodsFor: 'solving' stamp: 'mg 1/10/2013 22:53:53.085'!
initializeHighlightCandidates

	highlightCandidates := SudokuGrid new: 9.
	highlightCandidates indicesDo:   
		[:i :j | highlightCandidates at: i at: j put: OrderedCollection new].! !

!SudokuStrategy methodsFor: 'accessing' stamp: 'MG 12/15/2012 16:55'!
inputMatrix
	"Answer the value of inputMatrix"

	^ inputMatrix! !

!SudokuStrategy methodsFor: 'accessing' stamp: 'MG 12/15/2012 16:55'!
inputMatrix: anObject
	"Set the value of inputMatrix"

	inputMatrix := anObject! !

!SudokuStrategy methodsFor: 'accessing' stamp: 'MR 12/15/2012 15:55'!
removeCandidates
	"Answer the value of removeCandidates"

	^ removeCandidates! !

!SudokuStrategy methodsFor: 'accessing' stamp: 'MR 12/15/2012 15:56'!
removeCandidates: anObject
	"Set the value of removeCandidates"

	removeCandidates := anObject! !

!SudokuStrategy methodsFor: 'solving' stamp: 'mg 1/10/2013 22:41:22.205'!
solveFromCandidates: aMatrix
	inputMatrix := SudokuGrid newFrom: aMatrix.
	candidates := SudokuGrid newFrom: aMatrix.
	solvedCells := false.
	self
		initializeHighlightCandidates;
		checkStrategyForRows;
		checkStrategyForColumns;
		checkStrategyForBoxes;
		findRemoveCandidates;
		findHighlightCandidates.
	^ solvedCells! !

!SudokuStrategy methodsFor: 'accessing' stamp: 'MG 12/15/2012 16:55'!
solvedCells
	"Answer the value of solvedCells"

	^ solvedCells! !

!SudokuStrategy methodsFor: 'accessing' stamp: 'MG 12/15/2012 16:55'!
solvedCells: anObject
	"Set the value of solvedCells"

	solvedCells := anObject! !

!SudokuStrategy methodsFor: 'naming' stamp: 'MR 1/10/2013 15:25'!
strategyName
	self subclassResponsibility.! !

SudokuStrategy subclass: #SudokuStrategyBoxLineReduction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Strategies'!

!SudokuStrategyBoxLineReduction methodsFor: 'solving' stamp: 'mg 2/2/2013 15:53:04.256'!
checkStrategyForBoxes
"The strategy for boxes is called PointingPairs"
! !

!SudokuStrategyBoxLineReduction methodsFor: 'solving' stamp: 'mg 2/2/2013 17:10:12.093'!
checkStrategyForColumns
"This method searchs for numbers that can only be in one box, if we look at a column"
	1 to: inputMatrix columnCount do:
		[:column | 	| numberIndex numbersForReduction |
					numberIndex := self indecesPerNumberIn: (inputMatrix atColumn: column).  
					numbersForReduction := numberIndex select: 
						[:each | each size > 1 and: [each allSatisfy: [:number | (number-1 // 3) = (each first - 1 // 3)]]].
					numbersForReduction associationsDo: [:each |  | highlight box indices |
						highlight := false.
						indices := OrderedCollection new.
						indices add: (SudokuGrid boxIndexFromCellIndex: each value first @ column).
						indices add: (SudokuGrid boxIndexFromCellIndex: each value second @ column).
						each value size = 3 ifTrue: [indices add: (SudokuGrid boxIndexFromCellIndex: each value third @ column)]. 
					
						box := candidates atBox: (SudokuGrid boxNumberFromIndex: each value first @ column).
						box withIndexDo: [:candidateList :index | 
							(indices includes: index)
								ifFalse: [
									(candidateList includes: each key) ifTrue: [solvedCells := true. highlight := true].
									candidateList remove: each key ifAbsent: []	]].
						
						highlight ifTrue: [each value do: [:index |((highlightCandidates atColumn: column) at: index) add: each key]]]]! !

!SudokuStrategyBoxLineReduction methodsFor: 'solving' stamp: 'mg 2/2/2013 17:00:42.701'!
checkStrategyForRows
"This method searchs for numbers that can only be in one box, if we look at a row"
	1 to: inputMatrix rowCount do:
		[:row | 	| numberIndex numbersForReduction |
				numberIndex := self indecesPerNumberIn: (inputMatrix atRow: row).  
				numbersForReduction := numberIndex select: 
					[:each | each size > 1 and: [each allSatisfy: [:number | (number-1 // 3) = (each first - 1 // 3)]]].
				numbersForReduction associationsDo: [:each |  | highlight box indices |
					highlight := false.
					indices := OrderedCollection new.
					indices add: (SudokuGrid boxIndexFromCellIndex: row @ each value first).
					indices add: (SudokuGrid boxIndexFromCellIndex: row @ each value second).
					each value size = 3 ifTrue: [indices add: (SudokuGrid boxIndexFromCellIndex: row @ each value third)]. 
					
					box := candidates atBox: (SudokuGrid boxNumberFromIndex: row @ each value first).
					box withIndexDo: [:candidateList :index | 
						(indices includes: index)
							ifFalse: [
								(candidateList includes: each key) ifTrue: [solvedCells := true. highlight := true].
								candidateList remove: each key ifAbsent: []	]].
						
					highlight ifTrue: [each value do: [:index |((highlightCandidates atRow: row) at: index) add: each key]]].
					]! !

!SudokuStrategyBoxLineReduction methodsFor: 'naming' stamp: 'mg 2/2/2013 15:53:27.265'!
description
	^'Box Line Reduction'! !

!SudokuStrategyBoxLineReduction methodsFor: 'naming' stamp: 'mg 2/2/2013 15:53:33.225'!
strategyName
	^'Box Line Reduction'! !

SudokuStrategy subclass: #SudokuStrategyHiddenPairs
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Strategies'!

!SudokuStrategyHiddenPairs methodsFor: 'solving' stamp: 'mg 2/2/2013 11:57:57.802'!
checkStrategyForBoxes
"This method searches for pairs that have only two possible cells in a box."
	1 to: inputMatrix boxCount do:
		[:box| | numberIndex pairsDict |
				numberIndex := self indecesPerNumberIn: (inputMatrix atBox: box).  
				pairsDict := numberIndex select: [:each | each size = 2 and: [(numberIndex values occurrencesOf: each) > 1]].
				pairsDict size > 2 ifTrue: [^self]. "not yet implemented"
				pairsDict ifNotEmpty: [ | highlight |
					highlight := false.
					pairsDict values first do: [:index |
						((candidates atBox: box) at: index) size > 2
							ifTrue: [solvedCells := true. highlight := true]  .
						((candidates atBox: box) at: index) removeAllSuchThat: [:number | (pairsDict keys includes: number) not]].
					highlight ifTrue: [
							((highlightCandidates atBox: box) at: (pairsDict values first at: 1)) addAll: pairsDict keys.
							((highlightCandidates atBox: box) at: (pairsDict values first at: 2)) addAll: pairsDict keys]]].! !

!SudokuStrategyHiddenPairs methodsFor: 'solving' stamp: 'mg 2/2/2013 11:57:42.441'!
checkStrategyForColumns
"This method searches for pairs that have only two possible cells in a column."
	1 to: inputMatrix columnCount do:
		[:column| | numberIndex pairsDict |
				numberIndex := self indecesPerNumberIn: (inputMatrix atColumn: column).  
				pairsDict := numberIndex select: [:each | each size = 2 and: [(numberIndex values occurrencesOf: each) > 1]].
				pairsDict size > 2 ifTrue: [^self]. "not yet implemented"
				pairsDict ifNotEmpty: [ | highlight |
					highlight := false.
					pairsDict values first do: [:index |
						((candidates atColumn: column) at: index) size > 2
							ifTrue: [solvedCells := true. highlight := true]  .
						((candidates atColumn: column) at: index) removeAllSuchThat: [:number | (pairsDict keys includes: number) not]].
					highlight ifTrue: [
							(highlightCandidates at: (pairsDict values first at: 1) at: column) addAll: pairsDict keys.
							(highlightCandidates at: (pairsDict values first at: 2) at: column) addAll: pairsDict keys]]].! !

!SudokuStrategyHiddenPairs methodsFor: 'solving' stamp: 'mg 2/2/2013 11:57:51.17'!
checkStrategyForRows
"This method searches for pairs that have only two possible cells in a row."
	1 to: inputMatrix rowCount do:
		[:row | 	| numberIndex pairsDict |
				numberIndex := self indecesPerNumberIn: (inputMatrix atRow: row).  
				pairsDict := numberIndex select: [:each | each size = 2 and: [(numberIndex values occurrencesOf: each) > 1]].
				pairsDict size > 2 ifTrue: [^self]. "not yet implemented"
				pairsDict ifNotEmpty: [ | highlight |
						highlight := false.
						pairsDict values first do: [:index |
						((candidates atRow: row) at: index) size > 2
							ifTrue: [solvedCells := true. highlight := true].
						(candidates at: row at: index) removeAllSuchThat: [:number | (pairsDict keys includes: number) not]].
						highlight ifTrue: [
							(highlightCandidates at: row at: (pairsDict values first at: 1)) addAll: pairsDict keys.
							(highlightCandidates at: row at: (pairsDict values first at: 2)) addAll: pairsDict keys]]].! !

!SudokuStrategyHiddenPairs methodsFor: 'naming' stamp: 'mg 12/19/2012 14:58:43.844'!
description
	^ 'Hidden Pairs'! !

!SudokuStrategyHiddenPairs methodsFor: 'naming' stamp: 'MR 1/10/2013 15:27'!
strategyName
	^ 'Hidden Pairs'! !

SudokuStrategy subclass: #SudokuStrategyHiddenSingles
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Strategies'!

!SudokuStrategyHiddenSingles methodsFor: 'solving' stamp: 'mg 1/10/2013 14:35:23.891'!
checkStrategyForBoxes
"This method searches for numbers that have only one possible cell in a box. (i,j) are the indices of the top left cells in each box."
	1 to: 9 by: 3 do:
		[:i | 1 to: 9 by: 3 do:
			[:j | 1 to: 9 do:
				[:number | 	| index |  	
							index := self number: number hasOnePossibilityIn: (self getBoxCellsAt: i at: j in: inputMatrix ).
							(index = 0)
								ifFalse: [self removeCandidatesInCell: ((self getBoxCellsAt: i at: j in: candidates) at: index) 
											hiddenSingle: number.
										(highlightCandidates at: i+(index-1//3) at: j+(index-1\\3))add: number]]]].! !

!SudokuStrategyHiddenSingles methodsFor: 'solving' stamp: 'mg 1/10/2013 14:38:50.139'!
checkStrategyForColumns
"This method searches for numbers that have only one possible cell in a column."
	1 to: candidates columnCount do:
		[:column | 1 to: 9 do: 
			[:number |	| index | 	
						index := self number: number hasOnePossibilityIn: (inputMatrix atColumn: column).
						(index = 0)
							ifFalse: [self removeCandidatesInCell: ((candidates atColumn: column) at: index) 
										hiddenSingle: number. 
									(highlightCandidates at: index at: column) add: number]]].! !

!SudokuStrategyHiddenSingles methodsFor: 'solving' stamp: 'mg 1/10/2013 14:28:35.157'!
checkStrategyForRows
"This method searches for numbers that have only one possible cell in a row."
	1 to: candidates rowCount do:
		[:row | 1 to: 9 do: 
			[:number | 	| index | 	
						index := self number: number hasOnePossibilityIn: (inputMatrix atRow: row).
						(index = 0)
							ifFalse: [self removeCandidatesInCell: ((candidates atRow: row) at: index) 
										hiddenSingle: number. 
									(highlightCandidates at: row at: index) add: number]]].! !

!SudokuStrategyHiddenSingles methodsFor: 'naming' stamp: 'mg 12/19/2012 14:47:08.384'!
description
	^ 'Hidden Singles'! !

!SudokuStrategyHiddenSingles methodsFor: 'solving' stamp: 'mg 12/19/2012 14:00:37.973'!
number: aNumber hasOnePossibilityIn: aCollection
"This method checks whether aNumber can only be in one cell of aCollection. If that is the case, the index of that cell will be returned. Otherwise, this method returns 0."
	| numberOfPossibleCells indexOfCell |
	numberOfPossibleCells := 0.
	indexOfCell := 0.
	aCollection withIndexDo: 
		[:candidates :index | (candidates size = 1 and: [candidates includes: aNumber])
								ifTrue: [^ 0].  
							(candidates includes: aNumber)
								ifTrue: [	numberOfPossibleCells := numberOfPossibleCells + 1. 
										indexOfCell := index]].
					
	(numberOfPossibleCells = 1)
		ifFalse: [indexOfCell := 0].
		
	^ indexOfCell! !

!SudokuStrategyHiddenSingles methodsFor: 'solving' stamp: 'mg 12/19/2012 14:01:01.119'!
removeCandidatesInCell: aCollection hiddenSingle: aNumber
	solvedCells := solvedCells or: [aCollection size > 1].
	aCollection removeAllSuchThat: [:number | (number = aNumber) not].! !

!SudokuStrategyHiddenSingles methodsFor: 'naming' stamp: 'MR 1/10/2013 15:26'!
strategyName
	^ 'Hidden Singles'! !

SudokuStrategy subclass: #SudokuStrategyNakedPairs
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Strategies'!

!SudokuStrategyNakedPairs methodsFor: 'solving' stamp: 'MG 12/19/2012 12:13'!
checkStrategyForBoxAt: i at: j
	| box listOfPairs possiblePairs |
	box := self getBoxCellsAt: i at: j in: inputMatrix. 
	possiblePairs := self findCellsWithTwoPossibilitiesIn: box.
	listOfPairs := self searchForPairs: possiblePairs.
	listOfPairs notEmpty
		ifTrue: [self removeCandidatesForPairs: listOfPairs in: (self getBoxCellsAt: i at: j in: candidates)].! !

!SudokuStrategyNakedPairs methodsFor: 'solving' stamp: 'MG 12/19/2012 12:06'!
checkStrategyForBoxes
"(i,j) are the indices of the top left cells in each box."
	1 to: 9 by: 3 do:
		[:i | 1 to: 9 by: 3 do:
			[:j | self checkStrategyForBoxAt: i at: j]].! !

!SudokuStrategyNakedPairs methodsFor: 'solving' stamp: 'MG 12/19/2012 00:58'!
checkStrategyForColumns
	1 to: inputMatrix columnCount do:
		[:column | | possibleCells listOfPairs | 
				possibleCells := self findCellsWithTwoPossibilitiesIn: (inputMatrix atColumn: column).
				listOfPairs := self searchForPairs: possibleCells.
				listOfPairs notEmpty
					ifTrue: [self removeCandidatesForPairs: listOfPairs in: (candidates atColumn: column)]]! !

!SudokuStrategyNakedPairs methodsFor: 'solving' stamp: 'MG 12/19/2012 00:29'!
checkStrategyForRows
	1 to: inputMatrix rowCount do:
		[:row | | possibleCells listOfPairs | 
				possibleCells := self findCellsWithTwoPossibilitiesIn: (inputMatrix atRow: row).
				listOfPairs := self searchForPairs: possibleCells.
				listOfPairs notEmpty
					ifTrue: [self removeCandidatesForPairs: listOfPairs in: (candidates atRow: row)]]! !

!SudokuStrategyNakedPairs methodsFor: 'naming' stamp: 'mg 12/19/2012 14:46:54.76'!
description
	^ 'Naked Pairs'! !

!SudokuStrategyNakedPairs methodsFor: 'solving' stamp: 'MG 12/19/2012 00:29'!
findCellsWithTwoPossibilitiesIn: aCollection
	^ aCollection select: [:candidates | candidates size = 2].! !

!SudokuStrategyNakedPairs methodsFor: 'solving' stamp: 'mg 1/17/2013 16:06:00.943'!
findHighlightCandidates
	self 
		findHighlightCandidatesForRows;
		findHighlightCandidatesForColumns;
		findHighlightCandidatesForBoxes.! !

!SudokuStrategyNakedPairs methodsFor: 'solving' stamp: 'mg 1/17/2013 16:05:52.534'!
findHighlightCandidatesForBoxes
	1 to: inputMatrix boxCount do:
		[:boxIndex | | pairs | 
				pairs := OrderedCollection new.
				1 to: inputMatrix boxCount - 1 do:
					[:cellIndex | | candidateList cell |
							cell := SudokuGrid indexOfbox: boxIndex cell: cellIndex.
							candidateList := inputMatrix at: cell x at: cell y.
							candidateList size = 2
								ifTrue: [	(cellIndex+1 to: inputMatrix boxCount) 
											do: [:cellIndex2 | | cell2 |
															cell2 := SudokuGrid indexOfbox: boxIndex cell: cellIndex2.
															(candidateList includesAllOf: (inputMatrix at: cell2 x at: cell2 y))
																ifTrue: [pairs add: (OrderedCollection with: cellIndex with: cellIndex2)]]]].
				pairs do: 
					[:pairPositions | | pair | 
									pair := (inputMatrix atBox: boxIndex) at: (pairPositions at: 1).
									((removeCandidates atBox: boxIndex) contains: [:candidateList | candidateList includesAnyOf: pair])
										ifTrue: [ | firstPairIndex secondPairIndex |
												firstPairIndex := SudokuGrid indexOfbox: boxIndex cell: (pairPositions at: 1).
												secondPairIndex := SudokuGrid indexOfbox: boxIndex cell: (pairPositions at: 2).
												(highlightCandidates at: firstPairIndex x at: firstPairIndex y) add: (pair at: 1).
												(highlightCandidates at: firstPairIndex x at: firstPairIndex y) add: (pair at: 2).
												(highlightCandidates at: secondPairIndex x at: secondPairIndex y) add: (pair at: 1).
												(highlightCandidates at: secondPairIndex x at: secondPairIndex y) add: (pair at: 2)]]].! !

!SudokuStrategyNakedPairs methodsFor: 'solving' stamp: 'mg 1/17/2013 15:51:24.737'!
findHighlightCandidatesForColumns
	1 to: inputMatrix columnCount do:
		[:column | | pairs | 
				pairs := OrderedCollection new.
				1 to: inputMatrix rowCount - 1 do:
					[:row | | candidateList |
							candidateList := inputMatrix at: row at: column.
							candidateList size = 2
								ifTrue: [	(row+1 to: inputMatrix rowCount) 
											do: [:row2 | (candidateList includesAllOf: (inputMatrix at: row2 at: column))
																ifTrue: [pairs add: (OrderedCollection with: row with: row2)]]]].
				pairs do: 
					[:pairPositions | | pair | 
									pair := inputMatrix at: (pairPositions at: 1) at: column.
									((removeCandidates atColumn: column) contains: [:candidateList | candidateList includesAnyOf: pair])
										ifTrue: [(highlightCandidates at: (pairPositions at: 1) at: column) add: (pair at: 1).
												(highlightCandidates at: (pairPositions at: 1) at: column) add: (pair at: 2).
												(highlightCandidates at: (pairPositions at: 2) at: column) add: (pair at: 1).
												(highlightCandidates at: (pairPositions at: 2) at: column) add: (pair at: 2)]]].! !

!SudokuStrategyNakedPairs methodsFor: 'solving' stamp: 'mg 1/17/2013 15:45:06.721'!
findHighlightCandidatesForRows
	1 to: inputMatrix rowCount do:
		[:row | | pairs | 
				pairs := OrderedCollection new.
				1 to: inputMatrix columnCount - 1 do:
					[:column | | candidateList |
							candidateList := inputMatrix at: row at: column.
							candidateList size = 2
								ifTrue: [	(column+1 to: inputMatrix columnCount) 
											do: [:column2 | (candidateList includesAllOf: (inputMatrix at: row at: column2))
																ifTrue: [pairs add: (OrderedCollection with: column with: column2)]]]].
				pairs do: 
					[:pairPositions | | pair | 
									pair := inputMatrix at: row at: (pairPositions at: 1).
									((removeCandidates atRow: row) contains: [:candidateList | candidateList includesAnyOf: pair])
										ifTrue: [(highlightCandidates at: row at: (pairPositions at: 1)) add: (pair at: 1).
												(highlightCandidates at: row at: (pairPositions at: 1)) add: (pair at: 2).
												(highlightCandidates at: row at: (pairPositions at: 2)) add: (pair at: 1).
												(highlightCandidates at: row at: (pairPositions at: 2)) add: (pair at: 2)]]].! !

!SudokuStrategyNakedPairs methodsFor: 'solving' stamp: 'MG 12/19/2012 00:50'!
removeCandidatesForPairs: aCollectionOfPairs in: aCollectionOfCandidateLists
	aCollectionOfPairs do:
		[:pair | aCollectionOfCandidateLists do:
			[:candidates | (pair asSet = candidates asSet)
								ifFalse: [(candidates includesAnyOf: pair)
											ifTrue: [solvedCells := true.
													candidates removeAllFoundIn: pair]]]].! !

!SudokuStrategyNakedPairs methodsFor: 'solving' stamp: 'MG 12/19/2012 00:54'!
searchForPairs: aCollection
"This method returns a collection of all pairs in aCollection. A pair is found if two lists have the same candidates."
	| possiblePairs pairs |
	pairs := OrderedCollection new.
	possiblePairs := OrderedCollection new.
	aCollection do:
		[:candidates | |foundPair|
					foundPair := false.
					possiblePairs do:
						[:pair | (pair includesAllOf: candidates) 
									ifTrue: [pairs add: candidates. foundPair := true]].
					foundPair ifFalse: [possiblePairs add: candidates]].
	^ pairs! !

!SudokuStrategyNakedPairs methodsFor: 'naming' stamp: 'MR 1/10/2013 15:26'!
strategyName
	^ 'Naked Pairs'! !

SudokuStrategy subclass: #SudokuStrategyPointingPairs
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Strategies'!

!SudokuStrategyPointingPairs methodsFor: 'solving' stamp: 'mg 2/1/2013 18:11:43.867'!
checkStrategyForBoxes
"This method searchs for numbers that can only be in one row or column, if we look at one box"
	1 to: inputMatrix boxCount do: 
		[:box | 	| numberIndex pointingNumbersRow pointingNumbersColumn |
				numberIndex := self indecesPerNumberIn: (inputMatrix atBox: box).  
				pointingNumbersRow := numberIndex select: 
					[:each | each size > 1 and: [each allSatisfy: [:number | (number-1 // 3) = (each first - 1 // 3)]]].
				pointingNumbersColumn :=  numberIndex select: 
					[:each | each size > 1 and: [each allSatisfy: [:number | (number-1 \\ 3) = (each first - 1 \\ 3)]]].
				
				pointingNumbersRow associationsDo: [:each | | indices row highlight |
					highlight := false.
					indices := OrderedCollection new.
					indices add: (SudokuGrid indexOfbox: box cell: each value first) y.
					indices add: (SudokuGrid indexOfbox: box cell: each value second) y.
					each value size = 3 ifTrue: [indices add: (SudokuGrid indexOfbox: box cell: each value third) y].
					
					row := candidates atRow: (SudokuGrid indexOfbox: box cell: each value first) x.
					row withIndexDo: [:candidatesList :index |
						 (indices includes: index) ifFalse: [
							(candidatesList includes: each key) ifTrue: [solvedCells := true. highlight := true].  
							candidatesList remove: each key ifAbsent: []]].
					
					highlight ifTrue: [each value do: [:index |((highlightCandidates atBox: box) at: index) add: each key]]].
				
				pointingNumbersColumn associationsDo: [:each | | indices column highlight |
					highlight := false.
					indices := OrderedCollection new.
					indices add: (SudokuGrid indexOfbox: box cell: each value first) x.
					indices add: (SudokuGrid indexOfbox: box cell: each value second) x.
					each value size = 3 ifTrue: [indices add: (SudokuGrid indexOfbox: box cell: each value third) x].
					
					column := candidates atColumn: (SudokuGrid indexOfbox: box cell: each value first) y.
					column withIndexDo: [:candidatesList :index |
						 (indices includes: index) ifFalse: [
							(candidatesList includes: each key) ifTrue: [solvedCells := true. highlight := true].  
							candidatesList remove: each key ifAbsent: []]].
					
					highlight ifTrue: [each value do: [:index |((highlightCandidates atBox: box) at: index) add: each key]]].
		]! !

!SudokuStrategyPointingPairs methodsFor: 'solving' stamp: 'mg 2/1/2013 17:25:53.592'!
checkStrategyForColumns
"The strategy for columns is called BoxLineReduction"! !

!SudokuStrategyPointingPairs methodsFor: 'solving' stamp: 'mg 2/1/2013 17:26:04.153'!
checkStrategyForRows
"The strategy for rows is called BoxLineReduction"! !

!SudokuStrategyPointingPairs methodsFor: 'naming' stamp: 'mg 2/1/2013 17:28:24.865'!
description
	^'Pointing Pairs'! !

!SudokuStrategyPointingPairs methodsFor: 'naming' stamp: 'mg 2/1/2013 17:28:31.578'!
strategyName 
	^'Pointing Pairs'! !

SudokuStrategy subclass: #SudokuStrategyRulesChecker
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Strategies'!

!SudokuStrategyRulesChecker class methodsFor: 'as yet unclassified' stamp: 'mg 2/3/2013 12:11:55.052'!
finished: aSudokuGrid
"This method checks if the Sudoku grid is solved completely and correctly."
	(aSudokuGrid allSatisfy: [:candidateList | candidateList size = 1])  
		ifFalse: [^false]
		ifTrue: [^self isValid: aSudokuGrid]    ! !

!SudokuStrategyRulesChecker class methodsFor: 'as yet unclassified' stamp: 'mg 2/3/2013 01:55:48.729'!
isValid: aSudokuGrid
"This method checks the rules in a candidates grid."
	1 to: aSudokuGrid rowCount do:
		[:i |	| row column box | 
			row := (aSudokuGrid atRow: i).
			row detect: [:numberList | numberList isEmpty or: [numberList size = 1 and: [(row select: [:each | numberList includesAllOf: each]) size > 1]]]
				ifFound: [:temp | ^false]
				ifNone: [].
			
			column := aSudokuGrid atColumn: i.
			column detect: [:numberList | numberList isEmpty or: [numberList size = 1 and: [(column select: [:each | numberList includesAllOf: each]) size > 1]]]
				ifFound: [:temp | ^false]
				ifNone: [].
		  	
			box := aSudokuGrid atBox: i.
			box detect: [:numberList | numberList isEmpty or: [numberList size = 1 and: [(box select: [:each | numberList includesAllOf: each]) size > 1]]]
				ifFound: [:temp | ^false]
				ifNone: []].
	^ true! !

!SudokuStrategyRulesChecker methodsFor: 'solving' stamp: 'mg 12/19/2012 14:43:49.708'!
checkConstraintForBoxAt: i at: j
"This method searches all assigned numbers in the box at (i,j) and removes this numbers in all other candidate lists in the box."
	| assignedNumbers box |
	box := self getBoxCellsAt: i at: j in: inputMatrix.
	assignedNumbers := self collectAssignedNumbers: box.
	self removeAssignedNumbers: assignedNumbers from: (self getBoxCellsAt: i at: j in: candidates).! !

!SudokuStrategyRulesChecker methodsFor: 'solving' stamp: 'mg 12/19/2012 13:34:38.321'!
checkStrategyForBoxes
"(i,j) are the indices of the top left cells in each box."
	1 to: 9 by: 3 do:
		[:i | 1 to: 9 by: 3 do:
			[:j | self checkConstraintForBoxAt: i at: j]].! !

!SudokuStrategyRulesChecker methodsFor: 'solving' stamp: 'mg 12/19/2012 14:43:49.623'!
checkStrategyForColumns
"This method searches all assigned numbers per column and removes this numbers in all other candidate lists in the column."
	1 to: candidates columnCount do:
		[:column | |assignedNumbers| 
			assignedNumbers := self collectAssignedNumbers: (inputMatrix atColumn: column).
			self removeAssignedNumbers: assignedNumbers from: (candidates atColumn: column)].! !

!SudokuStrategyRulesChecker methodsFor: 'solving' stamp: 'mg 12/19/2012 14:43:49.8'!
checkStrategyForRows
"This method searches all assigned numbers per row and removes this numbers in all other candidate lists in the row."
	1 to: candidates rowCount do:
		[:row | |assignedNumbers| assignedNumbers := self collectAssignedNumbers: (inputMatrix atRow: row).
			self removeAssignedNumbers: assignedNumbers from: (candidates atRow: row)].! !

!SudokuStrategyRulesChecker methodsFor: 'solving' stamp: 'MG 11/24/2012 15:22'!
collectAssignedNumbers: aCollection
	| assignedNumbers |
	assignedNumbers := OrderedCollection new.
	aCollection collect: 
			[:candidateList | (candidateList size == 1) 
								ifTrue: [assignedNumbers add: candidateList first]].
	^ assignedNumbers ! !

!SudokuStrategyRulesChecker methodsFor: 'naming' stamp: 'mg 2/2/2013 20:26:27.583'!
description
	^ 'Check rules'! !

!SudokuStrategyRulesChecker methodsFor: 'solving' stamp: 'mg 1/10/2013 21:11:18.959'!
findHighlightCandidates
	self
		findHighlightCandidatesForRows;
		findHighlightCandidatesForColumns;
		findHighlightCandidatesForBoxes.! !

!SudokuStrategyRulesChecker methodsFor: 'solving' stamp: 'mg 1/10/2013 22:52:27.998'!
findHighlightCandidatesForBoxes
	1 to: removeCandidates boxCount do:
		[:box | | allNumbers | 	
				allNumbers := (removeCandidates atBox: box) flatten asSet. 
				(inputMatrix atBox: box) 
					select: [:cellCandidates | cellCandidates size = 1 and: [allNumbers includes: cellCandidates first]]
					thenDo: [:highlightNumber | 
							| cellIndexInBox | 
							cellIndexInBox := (inputMatrix atBox: box) indexOf: highlightNumber.  
							highlightCandidates 
								at: (SudokuGrid indexOfbox: box cell: cellIndexInBox) x   
								at: (SudokuGrid indexOfbox: box cell: cellIndexInBox) y 
								put: highlightNumber]]! !

!SudokuStrategyRulesChecker methodsFor: 'solving' stamp: 'mg 1/10/2013 17:26:45.353'!
findHighlightCandidatesForColumns
	1 to: removeCandidates columnCount do:
		[:column | | allNumbers | 	
				allNumbers := (removeCandidates atColumn: column) flatten asSet. 
				(inputMatrix atColumn: column) 
					select: [:cellCandidates | cellCandidates size = 1 and: [allNumbers includes: cellCandidates first]]
					thenDo: [:highlightNumber | 
							highlightCandidates 
								at: ((inputMatrix atColumn: column) indexOf: highlightNumber) 
								at: column
								put: highlightNumber]].! !

!SudokuStrategyRulesChecker methodsFor: 'solving' stamp: 'mg 1/10/2013 17:24:20.268'!
findHighlightCandidatesForRows	
	1 to: removeCandidates rowCount do:
		[:row | 	| allNumbers | 	
				allNumbers := (removeCandidates atRow: row) flatten asSet. 
				(inputMatrix atRow: row) 
					select: [:cellCandidates | cellCandidates size = 1 and: [allNumbers includes: cellCandidates first]]
					thenDo: [:highlightNumber | 
							highlightCandidates 
								at: row 
								at: ((inputMatrix atRow: row) indexOf: highlightNumber) 
								put: highlightNumber]].! !

!SudokuStrategyRulesChecker methodsFor: 'solving' stamp: 'mg 12/19/2012 14:43:49.54'!
removeAssignedNumbers: numbers from: aCollection
"If at least one candidate is removed, solvedCells is true."
	aCollection do: [:candidateList | (candidateList size == 1)
										ifFalse: [solvedCells := solvedCells or: [candidateList includesAnyOf: numbers]. 
											candidateList removeAllFoundIn: numbers]].! !

!SudokuStrategyRulesChecker methodsFor: 'naming' stamp: 'mg 2/2/2013 20:26:13.912'!
strategyName
	^ 'Check rules'! !

SudokuStrategy subclass: #SudokuStrategyYWing
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Strategies'!

!SudokuStrategyYWing methodsFor: 'solving' stamp: 'mg 2/2/2013 23:27:57.185'!
checkStrategyForBoxes
"not needed for Y-Wing"! !

!SudokuStrategyYWing methodsFor: 'solving' stamp: 'mg 2/2/2013 23:27:46.53'!
checkStrategyForColumns
"not needed for Y-Wing"! !

!SudokuStrategyYWing methodsFor: 'solving' stamp: 'mg 2/3/2013 15:30:35.165'!
checkStrategyForRows
"It is a really tough strategy!! :-)"
"This method search for thre connected field that has two possibilities with three different numbers. Each number should be placed in one of exactly two possible cells."
	inputMatrix withIndicesDo: [:candidateList :x :y | 
		candidateList size = 2 ifTrue: [ | possibleCellsRow possibleCellsBox possibleCellsColumn |
				possibleCellsRow := (inputMatrix atRow: x) 
					select: [:each | each size = 2 and: [(each difference: candidateList) size = 1]].
				possibleCellsBox := (inputMatrix atBox: (SudokuGrid boxNumberFromIndex: x@y)) 
					select: [:each | each size = 2 and: [(each difference: candidateList) size = 1]].   
				possibleCellsColumn := (inputMatrix atColumn: y) 
					select: [:each | each size = 2 and: [(each difference: candidateList) size = 1]].
				
				possibleCellsRow do: [:candidatesListRow | | sameNumber missingNumbers |
					sameNumber := (candidatesListRow intersection: candidateList) first.
					missingNumbers := (candidateList reject: [:number | number = sameNumber]) union: (candidatesListRow reject: [:number | number = sameNumber]).
					(possibleCellsBox select: [:each | each includesAllOf: missingNumbers])
						ifNotEmpty: ["All cells in the row of the box cell that lay in the box of the current cell (with candidatesListRow) and the cells in the current box that lay in the row of the row cell."
							| removeNumber currentColumn boxCell boxNumber boxIndex columns1 columns2 |
							removeNumber := (missingNumbers difference: candidateList) anyOne.
							currentColumn := (inputMatrix atRow: x) indexOf: candidatesListRow.    
							boxCell := (possibleCellsBox select: [:each | each includesAllOf: missingNumbers]) first.
							boxNumber := SudokuGrid boxNumberFromIndex: x@y.
							boxIndex := SudokuGrid indexOfbox: boxNumber cell: ((inputMatrix atBox: boxNumber) indexOf: boxCell).
							(boxIndex x = x) ifFalse: [  
								"Remove candidates in current box"
								columns1 := SudokuGrid missingLineNumbersInBoxForLine: y.
								columns1 do: [:each |
									((candidates at: x at: each) includes: removeNumber) ifTrue: [
												solvedCells := true. 
												(highlightCandidates at: x at: currentColumn) addAll: candidatesListRow.
												(highlightCandidates at: boxIndex x at: boxIndex y) addAll: missingNumbers.
												(highlightCandidates at: x at: y) addAll: candidateList.
												(candidates at: x at: each) remove: removeNumber]].
									
								"Remove candidates in the box of the row cell"
								columns2 := SudokuGrid missingLineNumbersInBoxForLine: currentColumn.
								columns2 add: currentColumn.
								columns2 do: [:each |
									((candidates at: boxIndex x at: each) includes: removeNumber) ifTrue: [
												solvedCells := true. 
												(highlightCandidates at: x at: currentColumn) addAll: candidatesListRow.
												(highlightCandidates at: boxIndex x at: boxIndex y) addAll: missingNumbers.
												(highlightCandidates at: x at: y) addAll: candidateList.
												(candidates at: boxIndex x at: each) remove: removeNumber]]]]
								
						ifEmpty: [(possibleCellsColumn select: [:each | each includesAllOf: missingNumbers])
									ifNotEmpty: ["One possible cell to remove a candidate" | removeNumber row column |
										removeNumber := (missingNumbers difference: candidateList) anyOne.
										column := (inputMatrix atRow: x) indexOf: candidatesListRow.
										row := (inputMatrix atColumn: y) indexOf: (possibleCellsColumn select: [:each | each includesAllOf: missingNumbers]) first.
										((candidates at: row at: column) includes: removeNumber) ifTrue: [
											solvedCells := true. 
											(highlightCandidates at: x at: column) addAll: candidatesListRow.
											(highlightCandidates at: row at: y) addAll: missingNumbers.
											(highlightCandidates at: x at: y) addAll: candidateList.
											(candidates at: row at: column) remove: removeNumber]]]].
							
				possibleCellsRow ifEmpty: [
					possibleCellsColumn do: [:candidateListColumn | | sameNumber missingNumbers |
						sameNumber := (candidateListColumn intersection: candidateList) first.
						missingNumbers := (candidateList reject: [:number | number = sameNumber]) union: (candidateListColumn reject: [:number | number = sameNumber]).
						(possibleCellsBox select: [:each | each includesAllOf: missingNumbers])
							ifNotEmpty: ["All cells in the column of the box cell that lay in the box of the current cell (with candidateListColumn) and the cells in the current box that lay in the column of the column cell."
								| removeNumber row boxIndex rows2 rows1 boxCell boxNumber |
								removeNumber := (missingNumbers difference: candidateList) anyOne.
								boxCell := (possibleCellsBox select: [:each | each includesAllOf: missingNumbers]) first.
								boxNumber := SudokuGrid boxNumberFromIndex: x@y.
								boxIndex := SudokuGrid indexOfbox: boxNumber cell: ((inputMatrix atBox: boxNumber) indexOf: boxCell).
								row := (inputMatrix atColumn: y) indexOf: candidateListColumn. 
								
								(boxIndex y = y) ifFalse: [   
									"Remove candidates in the box of the cell"
									rows1 := SudokuGrid missingLineNumbersInBoxForLine: row.
									rows1 add: row.
									rows1 do: [:each |								  
										((candidates at: each at: boxIndex y) includes: removeNumber) ifTrue: [
												solvedCells := true. 
												(highlightCandidates at: row at: y) addAll: candidateListColumn.
												(highlightCandidates at: boxIndex x at: boxIndex y) addAll: missingNumbers.
												(highlightCandidates at: x at: y) addAll: candidateList.
												(candidates at: each at: boxIndex y) remove: removeNumber]].
										
									"Remove candidates in current box"		
									rows2 := SudokuGrid missingLineNumbersInBoxForLine: x.
									rows2 do: [:each |
										((candidates at: each at: y) includes: removeNumber) ifTrue: [
												solvedCells := true. 
												(highlightCandidates at: row at: y) addAll: candidateListColumn.
												(highlightCandidates at: boxIndex x at: boxIndex y) addAll: missingNumbers.
												(highlightCandidates at: x at: y) addAll: candidateList.
												(candidates at: each at: y) remove: removeNumber]]]]]]
			].
		 ]  ! !

!SudokuStrategyYWing methodsFor: 'naming' stamp: 'mg 2/2/2013 21:16:21.665'!
description
	^'Y-Wing'! !

!SudokuStrategyYWing methodsFor: 'naming' stamp: 'mg 2/2/2013 21:16:26.636'!
strategyName
	^'Y-Wing'! !

UserInterface subclass: #SudokuGUIOutsideSumUi
	instanceVariableNames: 'topSum1 topSum2 topSum3 topSum4 topSum5 topSum6 topSum7 topSum8 topSum9 leftSum1 leftSum2 leftSum3 leftSum4 leftSum5 leftSum6 leftSum7 leftSum8 leftSum9 grid rightSum1 rightSum2 rightSum3 rightSum4 rightSum5 rightSum6 rightSum7 rightSum8 rightSum9 bottomSum1 bottomSum2 bottomSum3 bottomSum4 bottomSum5 bottomSum6 bottomSum7 bottomSum8 bottomSum9'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-GUI'!

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:27'!
bottomSum1
	"No description given."
	
	^ bottomSum1! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
bottomSum2
	"No description given."
	
	^ bottomSum2! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
bottomSum3
	"No description given."
	
	^ bottomSum3! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:27'!
bottomSum4
	"No description given."
	
	^ bottomSum4! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
bottomSum5
	"No description given."
	
	^ bottomSum5! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
bottomSum6
	"No description given."
	
	^ bottomSum6! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
bottomSum7
	"No description given."
	
	^ bottomSum7! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
bottomSum8
	"No description given."
	
	^ bottomSum8! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
bottomSum9
	"No description given."
	
	^ bottomSum9! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:27'!
grid
	"No description given."
	
	^ grid! !

!SudokuGUIOutsideSumUi methodsFor: 'ui - private' stamp: 'mg 6/25/2014 13:57'!
initializePanel
	"This code is auto-generated. DO NOT modify it because another generation step will overwrite all changes."
	
	| unnamedMorph1 unnamedMorph2 unnamedMorph3 unnamedMorph4 unnamedMorph5 unnamedMorph6 unnamedMorph7 unnamedMorph8 unnamedMorph9 |
	
	panel := UiContainer new.
	panel changeTableLayout.
	panel extent: (879@878).
	panel position: (0@0).
	panel hResizing: (#rigid).
	panel vResizing: (#rigid).
	panel minWidth: (2).
	panel minHeight: (2).
	panel cellInset: (0).
	panel cellPositioning: (#center).
	panel cellSpacing: (#none).
	panel layoutInset: (4).
	panel listCentering: (#topLeft).
	panel listDirection: (#topToBottom).
	panel listSpacing: (#none).
	panel maxCellSize: (1073741823).
	panel minCellSize: (0).
	panel wrapCentering: (#topLeft).
	panel wrapDirection: (#none).
	panel frameFractions: (0@0 corner: 1@1).
	panel frameOffsets: (0@0 corner: 0@0).
	panel balloonText: (nil).
	panel balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.722)).
	panel color: (Color white).
	panel borderWidth: (0).
	panel borderStyle2: (#simple).
	panel borderColor: (Color black).
	panel setProperty: #objectName toValue: 'panel'.
	unnamedMorph1 := UiContainer new.
	unnamedMorph1 changeTableLayout.
	unnamedMorph1 extent: (877@79).
	unnamedMorph1 position: (4@4).
	unnamedMorph1 hResizing: (#spaceFill).
	unnamedMorph1 vResizing: (#rigid).
	unnamedMorph1 minWidth: (2).
	unnamedMorph1 minHeight: (2).
	unnamedMorph1 cellInset: (2).
	unnamedMorph1 cellPositioning: (#center).
	unnamedMorph1 cellSpacing: (#none).
	unnamedMorph1 layoutInset: (2).
	unnamedMorph1 listCentering: (#topLeft).
	unnamedMorph1 listDirection: (#leftToRight).
	unnamedMorph1 listSpacing: (#none).
	unnamedMorph1 maxCellSize: (1073741823).
	unnamedMorph1 minCellSize: (0).
	unnamedMorph1 wrapCentering: (#topLeft).
	unnamedMorph1 wrapDirection: (#none).
	unnamedMorph1 frameFractions: (0@0 corner: 1@1).
	unnamedMorph1 frameOffsets: (0@0 corner: 0@0).
	unnamedMorph1 balloonText: (nil).
	unnamedMorph1 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.733)).
	unnamedMorph1 color: (Color transparent).
	unnamedMorph1 borderWidth: (0).
	unnamedMorph1 borderStyle2: (#simple).
	unnamedMorph1 borderColor: (Color black).
	unnamedMorph2 := SudokuGivenNumberCell new.
	unnamedMorph2 changeTableLayout.
	unnamedMorph2 extent: (85@75).
	unnamedMorph2 position: (6@6).
	unnamedMorph2 hResizing: (#rigid).
	unnamedMorph2 vResizing: (#rigid).
	unnamedMorph2 minWidth: (2).
	unnamedMorph2 minHeight: (2).
	unnamedMorph2 cellInset: (0).
	unnamedMorph2 cellPositioning: (#center).
	unnamedMorph2 cellSpacing: (#none).
	unnamedMorph2 layoutInset: (0).
	unnamedMorph2 listCentering: (#center).
	unnamedMorph2 listDirection: (#topToBottom).
	unnamedMorph2 listSpacing: (#none).
	unnamedMorph2 maxCellSize: (1073741823).
	unnamedMorph2 minCellSize: (0).
	unnamedMorph2 wrapCentering: (#center).
	unnamedMorph2 wrapDirection: (#none).
	unnamedMorph2 frameFractions: (0@0 corner: 1@1).
	unnamedMorph2 frameOffsets: (0@0 corner: 0@0).
	unnamedMorph2 balloonText: (nil).
	unnamedMorph2 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.741)).
	unnamedMorph1 addChild: unnamedMorph2.
	topSum1 := SudokuGivenNumberCell new.
	topSum1 changeTableLayout.
	topSum1 extent: (75@75).
	topSum1 position: (93@6).
	topSum1 hResizing: (#rigid).
	topSum1 vResizing: (#rigid).
	topSum1 minWidth: (2).
	topSum1 minHeight: (2).
	topSum1 cellInset: (0).
	topSum1 cellPositioning: (#center).
	topSum1 cellSpacing: (#none).
	topSum1 layoutInset: (0).
	topSum1 listCentering: (#center).
	topSum1 listDirection: (#topToBottom).
	topSum1 listSpacing: (#none).
	topSum1 maxCellSize: (1073741823).
	topSum1 minCellSize: (0).
	topSum1 wrapCentering: (#center).
	topSum1 wrapDirection: (#none).
	topSum1 frameFractions: (0@0 corner: 1@1).
	topSum1 frameOffsets: (0@0 corner: 0@0).
	topSum1 balloonText: (nil).
	topSum1 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.741)).
	topSum1 setProperty: #objectName toValue: 'topSum1'.
	unnamedMorph1 addChild: topSum1.
	topSum2 := SudokuGivenNumberCell new.
	topSum2 changeTableLayout.
	topSum2 extent: (75@75).
	topSum2 position: (170@6).
	topSum2 hResizing: (#rigid).
	topSum2 vResizing: (#rigid).
	topSum2 minWidth: (2).
	topSum2 minHeight: (2).
	topSum2 cellInset: (0).
	topSum2 cellPositioning: (#center).
	topSum2 cellSpacing: (#none).
	topSum2 layoutInset: (0).
	topSum2 listCentering: (#center).
	topSum2 listDirection: (#topToBottom).
	topSum2 listSpacing: (#none).
	topSum2 maxCellSize: (1073741823).
	topSum2 minCellSize: (0).
	topSum2 wrapCentering: (#center).
	topSum2 wrapDirection: (#none).
	topSum2 frameFractions: (0@0 corner: 1@1).
	topSum2 frameOffsets: (0@0 corner: 0@0).
	topSum2 balloonText: (nil).
	topSum2 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.741)).
	topSum2 setProperty: #objectName toValue: 'topSum2'.
	unnamedMorph1 addChild: topSum2.
	topSum3 := SudokuGivenNumberCell new.
	topSum3 changeTableLayout.
	topSum3 extent: (75@75).
	topSum3 position: (247@6).
	topSum3 hResizing: (#rigid).
	topSum3 vResizing: (#rigid).
	topSum3 minWidth: (2).
	topSum3 minHeight: (2).
	topSum3 cellInset: (0).
	topSum3 cellPositioning: (#center).
	topSum3 cellSpacing: (#none).
	topSum3 layoutInset: (0).
	topSum3 listCentering: (#center).
	topSum3 listDirection: (#topToBottom).
	topSum3 listSpacing: (#none).
	topSum3 maxCellSize: (1073741823).
	topSum3 minCellSize: (0).
	topSum3 wrapCentering: (#center).
	topSum3 wrapDirection: (#none).
	topSum3 frameFractions: (0@0 corner: 1@1).
	topSum3 frameOffsets: (0@0 corner: 0@0).
	topSum3 balloonText: (nil).
	topSum3 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.741)).
	topSum3 setProperty: #objectName toValue: 'topSum3'.
	unnamedMorph1 addChild: topSum3.
	topSum4 := SudokuGivenNumberCell new.
	topSum4 changeTableLayout.
	topSum4 extent: (79@75).
	topSum4 position: (324@6).
	topSum4 hResizing: (#rigid).
	topSum4 vResizing: (#rigid).
	topSum4 minWidth: (2).
	topSum4 minHeight: (2).
	topSum4 cellInset: (0).
	topSum4 cellPositioning: (#center).
	topSum4 cellSpacing: (#none).
	topSum4 layoutInset: (0).
	topSum4 listCentering: (#center).
	topSum4 listDirection: (#topToBottom).
	topSum4 listSpacing: (#none).
	topSum4 maxCellSize: (1073741823).
	topSum4 minCellSize: (0).
	topSum4 wrapCentering: (#center).
	topSum4 wrapDirection: (#none).
	topSum4 frameFractions: (0@0 corner: 1@1).
	topSum4 frameOffsets: (0@0 corner: 0@0).
	topSum4 balloonText: (nil).
	topSum4 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.741)).
	topSum4 setProperty: #objectName toValue: 'topSum4'.
	unnamedMorph1 addChild: topSum4.
	topSum5 := SudokuGivenNumberCell new.
	topSum5 changeTableLayout.
	topSum5 extent: (75@75).
	topSum5 position: (405@6).
	topSum5 hResizing: (#rigid).
	topSum5 vResizing: (#rigid).
	topSum5 minWidth: (2).
	topSum5 minHeight: (2).
	topSum5 cellInset: (0).
	topSum5 cellPositioning: (#center).
	topSum5 cellSpacing: (#none).
	topSum5 layoutInset: (0).
	topSum5 listCentering: (#center).
	topSum5 listDirection: (#topToBottom).
	topSum5 listSpacing: (#none).
	topSum5 maxCellSize: (1073741823).
	topSum5 minCellSize: (0).
	topSum5 wrapCentering: (#center).
	topSum5 wrapDirection: (#none).
	topSum5 frameFractions: (0@0 corner: 1@1).
	topSum5 frameOffsets: (0@0 corner: 0@0).
	topSum5 balloonText: (nil).
	topSum5 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.741)).
	topSum5 setProperty: #objectName toValue: 'topSum5'.
	unnamedMorph1 addChild: topSum5.
	topSum6 := SudokuGivenNumberCell new.
	topSum6 changeTableLayout.
	topSum6 extent: (75@75).
	topSum6 position: (482@6).
	topSum6 hResizing: (#rigid).
	topSum6 vResizing: (#rigid).
	topSum6 minWidth: (2).
	topSum6 minHeight: (2).
	topSum6 cellInset: (0).
	topSum6 cellPositioning: (#center).
	topSum6 cellSpacing: (#none).
	topSum6 layoutInset: (0).
	topSum6 listCentering: (#center).
	topSum6 listDirection: (#topToBottom).
	topSum6 listSpacing: (#none).
	topSum6 maxCellSize: (1073741823).
	topSum6 minCellSize: (0).
	topSum6 wrapCentering: (#center).
	topSum6 wrapDirection: (#none).
	topSum6 frameFractions: (0@0 corner: 1@1).
	topSum6 frameOffsets: (0@0 corner: 0@0).
	topSum6 balloonText: (nil).
	topSum6 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.741)).
	topSum6 setProperty: #objectName toValue: 'topSum6'.
	unnamedMorph1 addChild: topSum6.
	topSum7 := SudokuGivenNumberCell new.
	topSum7 changeTableLayout.
	topSum7 extent: (79@75).
	topSum7 position: (559@6).
	topSum7 hResizing: (#rigid).
	topSum7 vResizing: (#rigid).
	topSum7 minWidth: (2).
	topSum7 minHeight: (2).
	topSum7 cellInset: (0).
	topSum7 cellPositioning: (#center).
	topSum7 cellSpacing: (#none).
	topSum7 layoutInset: (0).
	topSum7 listCentering: (#center).
	topSum7 listDirection: (#topToBottom).
	topSum7 listSpacing: (#none).
	topSum7 maxCellSize: (1073741823).
	topSum7 minCellSize: (0).
	topSum7 wrapCentering: (#center).
	topSum7 wrapDirection: (#none).
	topSum7 frameFractions: (0@0 corner: 1@1).
	topSum7 frameOffsets: (0@0 corner: 0@0).
	topSum7 balloonText: (nil).
	topSum7 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.741)).
	topSum7 setProperty: #objectName toValue: 'topSum7'.
	unnamedMorph1 addChild: topSum7.
	topSum8 := SudokuGivenNumberCell new.
	topSum8 changeTableLayout.
	topSum8 extent: (75@75).
	topSum8 position: (640@6).
	topSum8 hResizing: (#rigid).
	topSum8 vResizing: (#rigid).
	topSum8 minWidth: (2).
	topSum8 minHeight: (2).
	topSum8 cellInset: (0).
	topSum8 cellPositioning: (#center).
	topSum8 cellSpacing: (#none).
	topSum8 layoutInset: (0).
	topSum8 listCentering: (#center).
	topSum8 listDirection: (#topToBottom).
	topSum8 listSpacing: (#none).
	topSum8 maxCellSize: (1073741823).
	topSum8 minCellSize: (0).
	topSum8 wrapCentering: (#center).
	topSum8 wrapDirection: (#none).
	topSum8 frameFractions: (0@0 corner: 1@1).
	topSum8 frameOffsets: (0@0 corner: 0@0).
	topSum8 balloonText: (nil).
	topSum8 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.741)).
	topSum8 setProperty: #objectName toValue: 'topSum8'.
	unnamedMorph1 addChild: topSum8.
	topSum9 := SudokuGivenNumberCell new.
	topSum9 changeTableLayout.
	topSum9 extent: (75@75).
	topSum9 position: (717@6).
	topSum9 hResizing: (#rigid).
	topSum9 vResizing: (#rigid).
	topSum9 minWidth: (2).
	topSum9 minHeight: (2).
	topSum9 cellInset: (0).
	topSum9 cellPositioning: (#center).
	topSum9 cellSpacing: (#none).
	topSum9 layoutInset: (0).
	topSum9 listCentering: (#center).
	topSum9 listDirection: (#topToBottom).
	topSum9 listSpacing: (#none).
	topSum9 maxCellSize: (1073741823).
	topSum9 minCellSize: (0).
	topSum9 wrapCentering: (#center).
	topSum9 wrapDirection: (#none).
	topSum9 frameFractions: (0@0 corner: 1@1).
	topSum9 frameOffsets: (0@0 corner: 0@0).
	topSum9 balloonText: (nil).
	topSum9 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.741)).
	topSum9 setProperty: #objectName toValue: 'topSum9'.
	unnamedMorph1 addChild: topSum9.
	unnamedMorph3 := SudokuGivenNumberCell new.
	unnamedMorph3 changeTableLayout.
	unnamedMorph3 extent: (85@75).
	unnamedMorph3 position: (794@6).
	unnamedMorph3 hResizing: (#rigid).
	unnamedMorph3 vResizing: (#rigid).
	unnamedMorph3 minWidth: (2).
	unnamedMorph3 minHeight: (2).
	unnamedMorph3 cellInset: (0).
	unnamedMorph3 cellPositioning: (#center).
	unnamedMorph3 cellSpacing: (#none).
	unnamedMorph3 layoutInset: (0).
	unnamedMorph3 listCentering: (#center).
	unnamedMorph3 listDirection: (#topToBottom).
	unnamedMorph3 listSpacing: (#none).
	unnamedMorph3 maxCellSize: (1073741823).
	unnamedMorph3 minCellSize: (0).
	unnamedMorph3 wrapCentering: (#center).
	unnamedMorph3 wrapDirection: (#none).
	unnamedMorph3 frameFractions: (0@0 corner: 1@1).
	unnamedMorph3 frameOffsets: (0@0 corner: 0@0).
	unnamedMorph3 balloonText: (nil).
	unnamedMorph3 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.741)).
	unnamedMorph1 addChild: unnamedMorph3.
	panel addChild: unnamedMorph1.
	unnamedMorph4 := UiContainer new.
	unnamedMorph4 changeTableLayout.
	unnamedMorph4 extent: (877@719).
	unnamedMorph4 position: (4@83).
	unnamedMorph4 hResizing: (#spaceFill).
	unnamedMorph4 vResizing: (#spaceFill).
	unnamedMorph4 minWidth: (2).
	unnamedMorph4 minHeight: (2).
	unnamedMorph4 cellInset: (0).
	unnamedMorph4 cellPositioning: (#center).
	unnamedMorph4 cellSpacing: (#none).
	unnamedMorph4 layoutInset: (2).
	unnamedMorph4 listCentering: (#topLeft).
	unnamedMorph4 listDirection: (#leftToRight).
	unnamedMorph4 listSpacing: (#none).
	unnamedMorph4 maxCellSize: (1073741823).
	unnamedMorph4 minCellSize: (0).
	unnamedMorph4 wrapCentering: (#topLeft).
	unnamedMorph4 wrapDirection: (#none).
	unnamedMorph4 frameFractions: (0@0 corner: 1@1).
	unnamedMorph4 frameOffsets: (0@0 corner: 0@0).
	unnamedMorph4 balloonText: (nil).
	unnamedMorph4 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.733)).
	unnamedMorph4 color: (Color transparent).
	unnamedMorph4 borderWidth: (0).
	unnamedMorph4 borderStyle2: (#simple).
	unnamedMorph4 borderColor: (Color black).
	unnamedMorph5 := UiContainer new.
	unnamedMorph5 changeTableLayout.
	unnamedMorph5 extent: (79@715).
	unnamedMorph5 position: (6@85).
	unnamedMorph5 hResizing: (#rigid).
	unnamedMorph5 vResizing: (#spaceFill).
	unnamedMorph5 minWidth: (2).
	unnamedMorph5 minHeight: (2).
	unnamedMorph5 cellInset: (2).
	unnamedMorph5 cellPositioning: (#center).
	unnamedMorph5 cellSpacing: (#none).
	unnamedMorph5 layoutInset: (4).
	unnamedMorph5 listCentering: (#topLeft).
	unnamedMorph5 listDirection: (#topToBottom).
	unnamedMorph5 listSpacing: (#none).
	unnamedMorph5 maxCellSize: (1073741823).
	unnamedMorph5 minCellSize: (0).
	unnamedMorph5 wrapCentering: (#topLeft).
	unnamedMorph5 wrapDirection: (#none).
	unnamedMorph5 frameFractions: (0@0 corner: 1@1).
	unnamedMorph5 frameOffsets: (0@0 corner: 0@0).
	unnamedMorph5 balloonText: (nil).
	unnamedMorph5 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.733)).
	unnamedMorph5 color: (Color transparent).
	unnamedMorph5 borderWidth: (0).
	unnamedMorph5 borderStyle2: (#simple).
	unnamedMorph5 borderColor: (Color black).
	leftSum1 := SudokuGivenNumberCell new.
	leftSum1 changeTableLayout.
	leftSum1 extent: (75@79).
	leftSum1 position: (10@89).
	leftSum1 hResizing: (#rigid).
	leftSum1 vResizing: (#rigid).
	leftSum1 minWidth: (2).
	leftSum1 minHeight: (2).
	leftSum1 cellInset: (0).
	leftSum1 cellPositioning: (#center).
	leftSum1 cellSpacing: (#none).
	leftSum1 layoutInset: (0).
	leftSum1 listCentering: (#center).
	leftSum1 listDirection: (#topToBottom).
	leftSum1 listSpacing: (#none).
	leftSum1 maxCellSize: (1073741823).
	leftSum1 minCellSize: (0).
	leftSum1 wrapCentering: (#center).
	leftSum1 wrapDirection: (#none).
	leftSum1 frameFractions: (0@0 corner: 1@1).
	leftSum1 frameOffsets: (0@0 corner: 0@0).
	leftSum1 balloonText: (nil).
	leftSum1 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.741)).
	leftSum1 setProperty: #objectName toValue: 'leftSum1'.
	unnamedMorph5 addChild: leftSum1.
	leftSum2 := SudokuGivenNumberCell new.
	leftSum2 changeTableLayout.
	leftSum2 extent: (75@75).
	leftSum2 position: (10@170).
	leftSum2 hResizing: (#rigid).
	leftSum2 vResizing: (#rigid).
	leftSum2 minWidth: (2).
	leftSum2 minHeight: (2).
	leftSum2 cellInset: (0).
	leftSum2 cellPositioning: (#center).
	leftSum2 cellSpacing: (#none).
	leftSum2 layoutInset: (0).
	leftSum2 listCentering: (#center).
	leftSum2 listDirection: (#topToBottom).
	leftSum2 listSpacing: (#none).
	leftSum2 maxCellSize: (1073741823).
	leftSum2 minCellSize: (0).
	leftSum2 wrapCentering: (#center).
	leftSum2 wrapDirection: (#none).
	leftSum2 frameFractions: (0@0 corner: 1@1).
	leftSum2 frameOffsets: (0@0 corner: 0@0).
	leftSum2 balloonText: (nil).
	leftSum2 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.741)).
	leftSum2 setProperty: #objectName toValue: 'leftSum2'.
	unnamedMorph5 addChild: leftSum2.
	leftSum3 := SudokuGivenNumberCell new.
	leftSum3 changeTableLayout.
	leftSum3 extent: (75@75).
	leftSum3 position: (10@247).
	leftSum3 hResizing: (#rigid).
	leftSum3 vResizing: (#rigid).
	leftSum3 minWidth: (2).
	leftSum3 minHeight: (2).
	leftSum3 cellInset: (0).
	leftSum3 cellPositioning: (#center).
	leftSum3 cellSpacing: (#none).
	leftSum3 layoutInset: (0).
	leftSum3 listCentering: (#center).
	leftSum3 listDirection: (#topToBottom).
	leftSum3 listSpacing: (#none).
	leftSum3 maxCellSize: (1073741823).
	leftSum3 minCellSize: (0).
	leftSum3 wrapCentering: (#center).
	leftSum3 wrapDirection: (#none).
	leftSum3 frameFractions: (0@0 corner: 1@1).
	leftSum3 frameOffsets: (0@0 corner: 0@0).
	leftSum3 balloonText: (nil).
	leftSum3 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.741)).
	leftSum3 setProperty: #objectName toValue: 'leftSum3'.
	unnamedMorph5 addChild: leftSum3.
	leftSum4 := SudokuGivenNumberCell new.
	leftSum4 changeTableLayout.
	leftSum4 extent: (75@79).
	leftSum4 position: (10@324).
	leftSum4 hResizing: (#rigid).
	leftSum4 vResizing: (#rigid).
	leftSum4 minWidth: (2).
	leftSum4 minHeight: (2).
	leftSum4 cellInset: (0).
	leftSum4 cellPositioning: (#center).
	leftSum4 cellSpacing: (#none).
	leftSum4 layoutInset: (0).
	leftSum4 listCentering: (#center).
	leftSum4 listDirection: (#topToBottom).
	leftSum4 listSpacing: (#none).
	leftSum4 maxCellSize: (1073741823).
	leftSum4 minCellSize: (0).
	leftSum4 wrapCentering: (#center).
	leftSum4 wrapDirection: (#none).
	leftSum4 frameFractions: (0@0 corner: 1@1).
	leftSum4 frameOffsets: (0@0 corner: 0@0).
	leftSum4 balloonText: (nil).
	leftSum4 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.741)).
	leftSum4 setProperty: #objectName toValue: 'leftSum4'.
	unnamedMorph5 addChild: leftSum4.
	leftSum5 := SudokuGivenNumberCell new.
	leftSum5 changeTableLayout.
	leftSum5 extent: (75@75).
	leftSum5 position: (10@405).
	leftSum5 hResizing: (#rigid).
	leftSum5 vResizing: (#rigid).
	leftSum5 minWidth: (2).
	leftSum5 minHeight: (2).
	leftSum5 cellInset: (0).
	leftSum5 cellPositioning: (#center).
	leftSum5 cellSpacing: (#none).
	leftSum5 layoutInset: (0).
	leftSum5 listCentering: (#center).
	leftSum5 listDirection: (#topToBottom).
	leftSum5 listSpacing: (#none).
	leftSum5 maxCellSize: (1073741823).
	leftSum5 minCellSize: (0).
	leftSum5 wrapCentering: (#center).
	leftSum5 wrapDirection: (#none).
	leftSum5 frameFractions: (0@0 corner: 1@1).
	leftSum5 frameOffsets: (0@0 corner: 0@0).
	leftSum5 balloonText: (nil).
	leftSum5 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.741)).
	leftSum5 setProperty: #objectName toValue: 'leftSum5'.
	unnamedMorph5 addChild: leftSum5.
	leftSum6 := SudokuGivenNumberCell new.
	leftSum6 changeTableLayout.
	leftSum6 extent: (75@75).
	leftSum6 position: (10@482).
	leftSum6 hResizing: (#rigid).
	leftSum6 vResizing: (#rigid).
	leftSum6 minWidth: (2).
	leftSum6 minHeight: (2).
	leftSum6 cellInset: (0).
	leftSum6 cellPositioning: (#center).
	leftSum6 cellSpacing: (#none).
	leftSum6 layoutInset: (0).
	leftSum6 listCentering: (#center).
	leftSum6 listDirection: (#topToBottom).
	leftSum6 listSpacing: (#none).
	leftSum6 maxCellSize: (1073741823).
	leftSum6 minCellSize: (0).
	leftSum6 wrapCentering: (#center).
	leftSum6 wrapDirection: (#none).
	leftSum6 frameFractions: (0@0 corner: 1@1).
	leftSum6 frameOffsets: (0@0 corner: 0@0).
	leftSum6 balloonText: (nil).
	leftSum6 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.741)).
	leftSum6 setProperty: #objectName toValue: 'leftSum6'.
	unnamedMorph5 addChild: leftSum6.
	leftSum7 := SudokuGivenNumberCell new.
	leftSum7 changeTableLayout.
	leftSum7 extent: (75@79).
	leftSum7 position: (10@559).
	leftSum7 hResizing: (#rigid).
	leftSum7 vResizing: (#rigid).
	leftSum7 minWidth: (2).
	leftSum7 minHeight: (2).
	leftSum7 cellInset: (0).
	leftSum7 cellPositioning: (#center).
	leftSum7 cellSpacing: (#none).
	leftSum7 layoutInset: (0).
	leftSum7 listCentering: (#center).
	leftSum7 listDirection: (#topToBottom).
	leftSum7 listSpacing: (#none).
	leftSum7 maxCellSize: (1073741823).
	leftSum7 minCellSize: (0).
	leftSum7 wrapCentering: (#center).
	leftSum7 wrapDirection: (#none).
	leftSum7 frameFractions: (0@0 corner: 1@1).
	leftSum7 frameOffsets: (0@0 corner: 0@0).
	leftSum7 balloonText: (nil).
	leftSum7 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.741)).
	leftSum7 setProperty: #objectName toValue: 'leftSum7'.
	unnamedMorph5 addChild: leftSum7.
	leftSum8 := SudokuGivenNumberCell new.
	leftSum8 changeTableLayout.
	leftSum8 extent: (75@75).
	leftSum8 position: (10@640).
	leftSum8 hResizing: (#rigid).
	leftSum8 vResizing: (#rigid).
	leftSum8 minWidth: (2).
	leftSum8 minHeight: (2).
	leftSum8 cellInset: (0).
	leftSum8 cellPositioning: (#center).
	leftSum8 cellSpacing: (#none).
	leftSum8 layoutInset: (0).
	leftSum8 listCentering: (#center).
	leftSum8 listDirection: (#topToBottom).
	leftSum8 listSpacing: (#none).
	leftSum8 maxCellSize: (1073741823).
	leftSum8 minCellSize: (0).
	leftSum8 wrapCentering: (#center).
	leftSum8 wrapDirection: (#none).
	leftSum8 frameFractions: (0@0 corner: 1@1).
	leftSum8 frameOffsets: (0@0 corner: 0@0).
	leftSum8 balloonText: (nil).
	leftSum8 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.741)).
	leftSum8 setProperty: #objectName toValue: 'leftSum8'.
	unnamedMorph5 addChild: leftSum8.
	leftSum9 := SudokuGivenNumberCell new.
	leftSum9 changeTableLayout.
	leftSum9 extent: (75@75).
	leftSum9 position: (10@717).
	leftSum9 hResizing: (#rigid).
	leftSum9 vResizing: (#rigid).
	leftSum9 minWidth: (2).
	leftSum9 minHeight: (2).
	leftSum9 cellInset: (0).
	leftSum9 cellPositioning: (#center).
	leftSum9 cellSpacing: (#none).
	leftSum9 layoutInset: (0).
	leftSum9 listCentering: (#center).
	leftSum9 listDirection: (#topToBottom).
	leftSum9 listSpacing: (#none).
	leftSum9 maxCellSize: (1073741823).
	leftSum9 minCellSize: (0).
	leftSum9 wrapCentering: (#center).
	leftSum9 wrapDirection: (#none).
	leftSum9 frameFractions: (0@0 corner: 1@1).
	leftSum9 frameOffsets: (0@0 corner: 0@0).
	leftSum9 balloonText: (nil).
	leftSum9 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.741)).
	leftSum9 setProperty: #objectName toValue: 'leftSum9'.
	unnamedMorph5 addChild: leftSum9.
	unnamedMorph4 addChild: unnamedMorph5.
	grid := SudokuGUI new.
	grid changeTableLayout.
	grid extent: (715@715).
	grid position: (85@85).
	grid hResizing: (#shrinkWrap).
	grid vResizing: (#shrinkWrap).
	grid minWidth: (2).
	grid minHeight: (2).
	grid cellInset: (0).
	grid cellPositioning: (#center).
	grid cellSpacing: (#none).
	grid layoutInset: (0).
	grid listCentering: (#topLeft).
	grid listDirection: (#leftToRight).
	grid listSpacing: (#none).
	grid maxCellSize: (1073741823).
	grid minCellSize: (0).
	grid wrapCentering: (#topLeft).
	grid wrapDirection: (#none).
	grid frameFractions: (0@0 corner: 1@1).
	grid frameOffsets: (0@0 corner: 0@0).
	grid balloonText: (nil).
	grid balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.733)).
	grid setProperty: #objectName toValue: 'grid'.
	unnamedMorph4 addChild: grid.
	unnamedMorph6 := UiContainer new.
	unnamedMorph6 changeTableLayout.
	unnamedMorph6 extent: (79@715).
	unnamedMorph6 position: (800@85).
	unnamedMorph6 hResizing: (#rigid).
	unnamedMorph6 vResizing: (#spaceFill).
	unnamedMorph6 minWidth: (2).
	unnamedMorph6 minHeight: (2).
	unnamedMorph6 cellInset: (2).
	unnamedMorph6 cellPositioning: (#center).
	unnamedMorph6 cellSpacing: (#none).
	unnamedMorph6 layoutInset: (0).
	unnamedMorph6 listCentering: (#topLeft).
	unnamedMorph6 listDirection: (#topToBottom).
	unnamedMorph6 listSpacing: (#none).
	unnamedMorph6 maxCellSize: (1073741823).
	unnamedMorph6 minCellSize: (0).
	unnamedMorph6 wrapCentering: (#topLeft).
	unnamedMorph6 wrapDirection: (#none).
	unnamedMorph6 frameFractions: (0@0 corner: 1@1).
	unnamedMorph6 frameOffsets: (0@0 corner: 0@0).
	unnamedMorph6 balloonText: (nil).
	unnamedMorph6 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.733)).
	unnamedMorph6 color: (Color transparent).
	unnamedMorph6 borderWidth: (0).
	unnamedMorph6 borderStyle2: (#simple).
	unnamedMorph6 borderColor: (Color black).
	rightSum1 := SudokuGivenNumberCell new.
	rightSum1 changeTableLayout.
	rightSum1 extent: (75@83).
	rightSum1 position: (800@85).
	rightSum1 hResizing: (#rigid).
	rightSum1 vResizing: (#rigid).
	rightSum1 minWidth: (2).
	rightSum1 minHeight: (2).
	rightSum1 cellInset: (0).
	rightSum1 cellPositioning: (#center).
	rightSum1 cellSpacing: (#none).
	rightSum1 layoutInset: (0).
	rightSum1 listCentering: (#center).
	rightSum1 listDirection: (#topToBottom).
	rightSum1 listSpacing: (#none).
	rightSum1 maxCellSize: (1073741823).
	rightSum1 minCellSize: (0).
	rightSum1 wrapCentering: (#center).
	rightSum1 wrapDirection: (#none).
	rightSum1 frameFractions: (0@0 corner: 1@1).
	rightSum1 frameOffsets: (0@0 corner: 0@0).
	rightSum1 balloonText: (nil).
	rightSum1 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.741)).
	rightSum1 setProperty: #objectName toValue: 'rightSum1'.
	unnamedMorph6 addChild: rightSum1.
	rightSum2 := SudokuGivenNumberCell new.
	rightSum2 changeTableLayout.
	rightSum2 extent: (75@75).
	rightSum2 position: (800@170).
	rightSum2 hResizing: (#rigid).
	rightSum2 vResizing: (#rigid).
	rightSum2 minWidth: (2).
	rightSum2 minHeight: (2).
	rightSum2 cellInset: (0).
	rightSum2 cellPositioning: (#center).
	rightSum2 cellSpacing: (#none).
	rightSum2 layoutInset: (0).
	rightSum2 listCentering: (#center).
	rightSum2 listDirection: (#topToBottom).
	rightSum2 listSpacing: (#none).
	rightSum2 maxCellSize: (1073741823).
	rightSum2 minCellSize: (0).
	rightSum2 wrapCentering: (#center).
	rightSum2 wrapDirection: (#none).
	rightSum2 frameFractions: (0@0 corner: 1@1).
	rightSum2 frameOffsets: (0@0 corner: 0@0).
	rightSum2 balloonText: (nil).
	rightSum2 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.741)).
	rightSum2 setProperty: #objectName toValue: 'rightSum2'.
	unnamedMorph6 addChild: rightSum2.
	rightSum3 := SudokuGivenNumberCell new.
	rightSum3 changeTableLayout.
	rightSum3 extent: (75@75).
	rightSum3 position: (800@247).
	rightSum3 hResizing: (#rigid).
	rightSum3 vResizing: (#rigid).
	rightSum3 minWidth: (2).
	rightSum3 minHeight: (2).
	rightSum3 cellInset: (0).
	rightSum3 cellPositioning: (#center).
	rightSum3 cellSpacing: (#none).
	rightSum3 layoutInset: (0).
	rightSum3 listCentering: (#center).
	rightSum3 listDirection: (#topToBottom).
	rightSum3 listSpacing: (#none).
	rightSum3 maxCellSize: (1073741823).
	rightSum3 minCellSize: (0).
	rightSum3 wrapCentering: (#center).
	rightSum3 wrapDirection: (#none).
	rightSum3 frameFractions: (0@0 corner: 1@1).
	rightSum3 frameOffsets: (0@0 corner: 0@0).
	rightSum3 balloonText: (nil).
	rightSum3 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.741)).
	rightSum3 setProperty: #objectName toValue: 'rightSum3'.
	unnamedMorph6 addChild: rightSum3.
	rightSum4 := SudokuGivenNumberCell new.
	rightSum4 changeTableLayout.
	rightSum4 extent: (75@79).
	rightSum4 position: (800@324).
	rightSum4 hResizing: (#rigid).
	rightSum4 vResizing: (#rigid).
	rightSum4 minWidth: (2).
	rightSum4 minHeight: (2).
	rightSum4 cellInset: (0).
	rightSum4 cellPositioning: (#center).
	rightSum4 cellSpacing: (#none).
	rightSum4 layoutInset: (0).
	rightSum4 listCentering: (#center).
	rightSum4 listDirection: (#topToBottom).
	rightSum4 listSpacing: (#none).
	rightSum4 maxCellSize: (1073741823).
	rightSum4 minCellSize: (0).
	rightSum4 wrapCentering: (#center).
	rightSum4 wrapDirection: (#none).
	rightSum4 frameFractions: (0@0 corner: 1@1).
	rightSum4 frameOffsets: (0@0 corner: 0@0).
	rightSum4 balloonText: (nil).
	rightSum4 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.741)).
	rightSum4 setProperty: #objectName toValue: 'rightSum4'.
	unnamedMorph6 addChild: rightSum4.
	rightSum5 := SudokuGivenNumberCell new.
	rightSum5 changeTableLayout.
	rightSum5 extent: (75@75).
	rightSum5 position: (800@405).
	rightSum5 hResizing: (#rigid).
	rightSum5 vResizing: (#rigid).
	rightSum5 minWidth: (2).
	rightSum5 minHeight: (2).
	rightSum5 cellInset: (0).
	rightSum5 cellPositioning: (#center).
	rightSum5 cellSpacing: (#none).
	rightSum5 layoutInset: (0).
	rightSum5 listCentering: (#center).
	rightSum5 listDirection: (#topToBottom).
	rightSum5 listSpacing: (#none).
	rightSum5 maxCellSize: (1073741823).
	rightSum5 minCellSize: (0).
	rightSum5 wrapCentering: (#center).
	rightSum5 wrapDirection: (#none).
	rightSum5 frameFractions: (0@0 corner: 1@1).
	rightSum5 frameOffsets: (0@0 corner: 0@0).
	rightSum5 balloonText: (nil).
	rightSum5 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.741)).
	rightSum5 setProperty: #objectName toValue: 'rightSum5'.
	unnamedMorph6 addChild: rightSum5.
	rightSum6 := SudokuGivenNumberCell new.
	rightSum6 changeTableLayout.
	rightSum6 extent: (75@75).
	rightSum6 position: (800@482).
	rightSum6 hResizing: (#rigid).
	rightSum6 vResizing: (#rigid).
	rightSum6 minWidth: (2).
	rightSum6 minHeight: (2).
	rightSum6 cellInset: (0).
	rightSum6 cellPositioning: (#center).
	rightSum6 cellSpacing: (#none).
	rightSum6 layoutInset: (0).
	rightSum6 listCentering: (#center).
	rightSum6 listDirection: (#topToBottom).
	rightSum6 listSpacing: (#none).
	rightSum6 maxCellSize: (1073741823).
	rightSum6 minCellSize: (0).
	rightSum6 wrapCentering: (#center).
	rightSum6 wrapDirection: (#none).
	rightSum6 frameFractions: (0@0 corner: 1@1).
	rightSum6 frameOffsets: (0@0 corner: 0@0).
	rightSum6 balloonText: (nil).
	rightSum6 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.741)).
	rightSum6 setProperty: #objectName toValue: 'rightSum6'.
	unnamedMorph6 addChild: rightSum6.
	rightSum7 := SudokuGivenNumberCell new.
	rightSum7 changeTableLayout.
	rightSum7 extent: (75@79).
	rightSum7 position: (800@559).
	rightSum7 hResizing: (#rigid).
	rightSum7 vResizing: (#rigid).
	rightSum7 minWidth: (2).
	rightSum7 minHeight: (2).
	rightSum7 cellInset: (0).
	rightSum7 cellPositioning: (#center).
	rightSum7 cellSpacing: (#none).
	rightSum7 layoutInset: (0).
	rightSum7 listCentering: (#center).
	rightSum7 listDirection: (#topToBottom).
	rightSum7 listSpacing: (#none).
	rightSum7 maxCellSize: (1073741823).
	rightSum7 minCellSize: (0).
	rightSum7 wrapCentering: (#center).
	rightSum7 wrapDirection: (#none).
	rightSum7 frameFractions: (0@0 corner: 1@1).
	rightSum7 frameOffsets: (0@0 corner: 0@0).
	rightSum7 balloonText: (nil).
	rightSum7 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.741)).
	rightSum7 setProperty: #objectName toValue: 'rightSum7'.
	unnamedMorph6 addChild: rightSum7.
	rightSum8 := SudokuGivenNumberCell new.
	rightSum8 changeTableLayout.
	rightSum8 extent: (75@75).
	rightSum8 position: (800@640).
	rightSum8 hResizing: (#rigid).
	rightSum8 vResizing: (#rigid).
	rightSum8 minWidth: (2).
	rightSum8 minHeight: (2).
	rightSum8 cellInset: (0).
	rightSum8 cellPositioning: (#center).
	rightSum8 cellSpacing: (#none).
	rightSum8 layoutInset: (0).
	rightSum8 listCentering: (#center).
	rightSum8 listDirection: (#topToBottom).
	rightSum8 listSpacing: (#none).
	rightSum8 maxCellSize: (1073741823).
	rightSum8 minCellSize: (0).
	rightSum8 wrapCentering: (#center).
	rightSum8 wrapDirection: (#none).
	rightSum8 frameFractions: (0@0 corner: 1@1).
	rightSum8 frameOffsets: (0@0 corner: 0@0).
	rightSum8 balloonText: (nil).
	rightSum8 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.741)).
	rightSum8 setProperty: #objectName toValue: 'rightSum8'.
	unnamedMorph6 addChild: rightSum8.
	rightSum9 := SudokuGivenNumberCell new.
	rightSum9 changeTableLayout.
	rightSum9 extent: (75@75).
	rightSum9 position: (800@717).
	rightSum9 hResizing: (#rigid).
	rightSum9 vResizing: (#rigid).
	rightSum9 minWidth: (2).
	rightSum9 minHeight: (2).
	rightSum9 cellInset: (0).
	rightSum9 cellPositioning: (#center).
	rightSum9 cellSpacing: (#none).
	rightSum9 layoutInset: (0).
	rightSum9 listCentering: (#center).
	rightSum9 listDirection: (#topToBottom).
	rightSum9 listSpacing: (#none).
	rightSum9 maxCellSize: (1073741823).
	rightSum9 minCellSize: (0).
	rightSum9 wrapCentering: (#center).
	rightSum9 wrapDirection: (#none).
	rightSum9 frameFractions: (0@0 corner: 1@1).
	rightSum9 frameOffsets: (0@0 corner: 0@0).
	rightSum9 balloonText: (nil).
	rightSum9 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.741)).
	rightSum9 setProperty: #objectName toValue: 'rightSum9'.
	unnamedMorph6 addChild: rightSum9.
	unnamedMorph4 addChild: unnamedMorph6.
	panel addChild: unnamedMorph4.
	unnamedMorph7 := UiContainer new.
	unnamedMorph7 changeTableLayout.
	unnamedMorph7 extent: (877@79).
	unnamedMorph7 position: (4@802).
	unnamedMorph7 hResizing: (#spaceFill).
	unnamedMorph7 vResizing: (#rigid).
	unnamedMorph7 minWidth: (2).
	unnamedMorph7 minHeight: (2).
	unnamedMorph7 cellInset: (2).
	unnamedMorph7 cellPositioning: (#center).
	unnamedMorph7 cellSpacing: (#none).
	unnamedMorph7 layoutInset: (2).
	unnamedMorph7 listCentering: (#topLeft).
	unnamedMorph7 listDirection: (#leftToRight).
	unnamedMorph7 listSpacing: (#none).
	unnamedMorph7 maxCellSize: (1073741823).
	unnamedMorph7 minCellSize: (0).
	unnamedMorph7 wrapCentering: (#topLeft).
	unnamedMorph7 wrapDirection: (#none).
	unnamedMorph7 frameFractions: (0@0 corner: 1@1).
	unnamedMorph7 frameOffsets: (0@0 corner: 0@0).
	unnamedMorph7 balloonText: (nil).
	unnamedMorph7 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.729)).
	unnamedMorph7 color: (Color transparent).
	unnamedMorph7 borderWidth: (0).
	unnamedMorph7 borderStyle2: (#simple).
	unnamedMorph7 borderColor: (Color black).
	unnamedMorph8 := SudokuGivenNumberCell new.
	unnamedMorph8 changeTableLayout.
	unnamedMorph8 extent: (85@75).
	unnamedMorph8 position: (6@804).
	unnamedMorph8 hResizing: (#rigid).
	unnamedMorph8 vResizing: (#rigid).
	unnamedMorph8 minWidth: (2).
	unnamedMorph8 minHeight: (2).
	unnamedMorph8 cellInset: (0).
	unnamedMorph8 cellPositioning: (#center).
	unnamedMorph8 cellSpacing: (#none).
	unnamedMorph8 layoutInset: (0).
	unnamedMorph8 listCentering: (#center).
	unnamedMorph8 listDirection: (#topToBottom).
	unnamedMorph8 listSpacing: (#none).
	unnamedMorph8 maxCellSize: (1073741823).
	unnamedMorph8 minCellSize: (0).
	unnamedMorph8 wrapCentering: (#center).
	unnamedMorph8 wrapDirection: (#none).
	unnamedMorph8 frameFractions: (0@0 corner: 1@1).
	unnamedMorph8 frameOffsets: (0@0 corner: 0@0).
	unnamedMorph8 balloonText: (nil).
	unnamedMorph8 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.733)).
	unnamedMorph7 addChild: unnamedMorph8.
	bottomSum1 := SudokuGivenNumberCell new.
	bottomSum1 changeTableLayout.
	bottomSum1 extent: (75@75).
	bottomSum1 position: (93@804).
	bottomSum1 hResizing: (#rigid).
	bottomSum1 vResizing: (#rigid).
	bottomSum1 minWidth: (2).
	bottomSum1 minHeight: (2).
	bottomSum1 cellInset: (0).
	bottomSum1 cellPositioning: (#center).
	bottomSum1 cellSpacing: (#none).
	bottomSum1 layoutInset: (0).
	bottomSum1 listCentering: (#center).
	bottomSum1 listDirection: (#topToBottom).
	bottomSum1 listSpacing: (#none).
	bottomSum1 maxCellSize: (1073741823).
	bottomSum1 minCellSize: (0).
	bottomSum1 wrapCentering: (#center).
	bottomSum1 wrapDirection: (#none).
	bottomSum1 frameFractions: (0@0 corner: 1@1).
	bottomSum1 frameOffsets: (0@0 corner: 0@0).
	bottomSum1 balloonText: (nil).
	bottomSum1 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.737)).
	bottomSum1 setProperty: #objectName toValue: 'bottomSum1'.
	unnamedMorph7 addChild: bottomSum1.
	bottomSum2 := SudokuGivenNumberCell new.
	bottomSum2 changeTableLayout.
	bottomSum2 extent: (75@75).
	bottomSum2 position: (170@804).
	bottomSum2 hResizing: (#rigid).
	bottomSum2 vResizing: (#rigid).
	bottomSum2 minWidth: (2).
	bottomSum2 minHeight: (2).
	bottomSum2 cellInset: (0).
	bottomSum2 cellPositioning: (#center).
	bottomSum2 cellSpacing: (#none).
	bottomSum2 layoutInset: (0).
	bottomSum2 listCentering: (#center).
	bottomSum2 listDirection: (#topToBottom).
	bottomSum2 listSpacing: (#none).
	bottomSum2 maxCellSize: (1073741823).
	bottomSum2 minCellSize: (0).
	bottomSum2 wrapCentering: (#center).
	bottomSum2 wrapDirection: (#none).
	bottomSum2 frameFractions: (0@0 corner: 1@1).
	bottomSum2 frameOffsets: (0@0 corner: 0@0).
	bottomSum2 balloonText: (nil).
	bottomSum2 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.737)).
	bottomSum2 setProperty: #objectName toValue: 'bottomSum2'.
	unnamedMorph7 addChild: bottomSum2.
	bottomSum3 := SudokuGivenNumberCell new.
	bottomSum3 changeTableLayout.
	bottomSum3 extent: (75@75).
	bottomSum3 position: (247@804).
	bottomSum3 hResizing: (#rigid).
	bottomSum3 vResizing: (#rigid).
	bottomSum3 minWidth: (2).
	bottomSum3 minHeight: (2).
	bottomSum3 cellInset: (0).
	bottomSum3 cellPositioning: (#center).
	bottomSum3 cellSpacing: (#none).
	bottomSum3 layoutInset: (0).
	bottomSum3 listCentering: (#center).
	bottomSum3 listDirection: (#topToBottom).
	bottomSum3 listSpacing: (#none).
	bottomSum3 maxCellSize: (1073741823).
	bottomSum3 minCellSize: (0).
	bottomSum3 wrapCentering: (#center).
	bottomSum3 wrapDirection: (#none).
	bottomSum3 frameFractions: (0@0 corner: 1@1).
	bottomSum3 frameOffsets: (0@0 corner: 0@0).
	bottomSum3 balloonText: (nil).
	bottomSum3 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.737)).
	bottomSum3 setProperty: #objectName toValue: 'bottomSum3'.
	unnamedMorph7 addChild: bottomSum3.
	bottomSum4 := SudokuGivenNumberCell new.
	bottomSum4 changeTableLayout.
	bottomSum4 extent: (79@75).
	bottomSum4 position: (324@804).
	bottomSum4 hResizing: (#rigid).
	bottomSum4 vResizing: (#rigid).
	bottomSum4 minWidth: (2).
	bottomSum4 minHeight: (2).
	bottomSum4 cellInset: (0).
	bottomSum4 cellPositioning: (#center).
	bottomSum4 cellSpacing: (#none).
	bottomSum4 layoutInset: (0).
	bottomSum4 listCentering: (#center).
	bottomSum4 listDirection: (#topToBottom).
	bottomSum4 listSpacing: (#none).
	bottomSum4 maxCellSize: (1073741823).
	bottomSum4 minCellSize: (0).
	bottomSum4 wrapCentering: (#center).
	bottomSum4 wrapDirection: (#none).
	bottomSum4 frameFractions: (0@0 corner: 1@1).
	bottomSum4 frameOffsets: (0@0 corner: 0@0).
	bottomSum4 balloonText: (nil).
	bottomSum4 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.737)).
	bottomSum4 setProperty: #objectName toValue: 'bottomSum4'.
	unnamedMorph7 addChild: bottomSum4.
	bottomSum5 := SudokuGivenNumberCell new.
	bottomSum5 changeTableLayout.
	bottomSum5 extent: (75@75).
	bottomSum5 position: (405@804).
	bottomSum5 hResizing: (#rigid).
	bottomSum5 vResizing: (#rigid).
	bottomSum5 minWidth: (2).
	bottomSum5 minHeight: (2).
	bottomSum5 cellInset: (0).
	bottomSum5 cellPositioning: (#center).
	bottomSum5 cellSpacing: (#none).
	bottomSum5 layoutInset: (0).
	bottomSum5 listCentering: (#center).
	bottomSum5 listDirection: (#topToBottom).
	bottomSum5 listSpacing: (#none).
	bottomSum5 maxCellSize: (1073741823).
	bottomSum5 minCellSize: (0).
	bottomSum5 wrapCentering: (#center).
	bottomSum5 wrapDirection: (#none).
	bottomSum5 frameFractions: (0@0 corner: 1@1).
	bottomSum5 frameOffsets: (0@0 corner: 0@0).
	bottomSum5 balloonText: (nil).
	bottomSum5 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.737)).
	bottomSum5 setProperty: #objectName toValue: 'bottomSum5'.
	unnamedMorph7 addChild: bottomSum5.
	bottomSum6 := SudokuGivenNumberCell new.
	bottomSum6 changeTableLayout.
	bottomSum6 extent: (75@75).
	bottomSum6 position: (482@804).
	bottomSum6 hResizing: (#rigid).
	bottomSum6 vResizing: (#rigid).
	bottomSum6 minWidth: (2).
	bottomSum6 minHeight: (2).
	bottomSum6 cellInset: (0).
	bottomSum6 cellPositioning: (#center).
	bottomSum6 cellSpacing: (#none).
	bottomSum6 layoutInset: (0).
	bottomSum6 listCentering: (#center).
	bottomSum6 listDirection: (#topToBottom).
	bottomSum6 listSpacing: (#none).
	bottomSum6 maxCellSize: (1073741823).
	bottomSum6 minCellSize: (0).
	bottomSum6 wrapCentering: (#center).
	bottomSum6 wrapDirection: (#none).
	bottomSum6 frameFractions: (0@0 corner: 1@1).
	bottomSum6 frameOffsets: (0@0 corner: 0@0).
	bottomSum6 balloonText: (nil).
	bottomSum6 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.737)).
	bottomSum6 setProperty: #objectName toValue: 'bottomSum6'.
	unnamedMorph7 addChild: bottomSum6.
	bottomSum7 := SudokuGivenNumberCell new.
	bottomSum7 changeTableLayout.
	bottomSum7 extent: (79@75).
	bottomSum7 position: (559@804).
	bottomSum7 hResizing: (#rigid).
	bottomSum7 vResizing: (#rigid).
	bottomSum7 minWidth: (2).
	bottomSum7 minHeight: (2).
	bottomSum7 cellInset: (0).
	bottomSum7 cellPositioning: (#center).
	bottomSum7 cellSpacing: (#none).
	bottomSum7 layoutInset: (0).
	bottomSum7 listCentering: (#center).
	bottomSum7 listDirection: (#topToBottom).
	bottomSum7 listSpacing: (#none).
	bottomSum7 maxCellSize: (1073741823).
	bottomSum7 minCellSize: (0).
	bottomSum7 wrapCentering: (#center).
	bottomSum7 wrapDirection: (#none).
	bottomSum7 frameFractions: (0@0 corner: 1@1).
	bottomSum7 frameOffsets: (0@0 corner: 0@0).
	bottomSum7 balloonText: (nil).
	bottomSum7 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.737)).
	bottomSum7 setProperty: #objectName toValue: 'bottomSum7'.
	unnamedMorph7 addChild: bottomSum7.
	bottomSum8 := SudokuGivenNumberCell new.
	bottomSum8 changeTableLayout.
	bottomSum8 extent: (75@75).
	bottomSum8 position: (640@804).
	bottomSum8 hResizing: (#rigid).
	bottomSum8 vResizing: (#rigid).
	bottomSum8 minWidth: (2).
	bottomSum8 minHeight: (2).
	bottomSum8 cellInset: (0).
	bottomSum8 cellPositioning: (#center).
	bottomSum8 cellSpacing: (#none).
	bottomSum8 layoutInset: (0).
	bottomSum8 listCentering: (#center).
	bottomSum8 listDirection: (#topToBottom).
	bottomSum8 listSpacing: (#none).
	bottomSum8 maxCellSize: (1073741823).
	bottomSum8 minCellSize: (0).
	bottomSum8 wrapCentering: (#center).
	bottomSum8 wrapDirection: (#none).
	bottomSum8 frameFractions: (0@0 corner: 1@1).
	bottomSum8 frameOffsets: (0@0 corner: 0@0).
	bottomSum8 balloonText: (nil).
	bottomSum8 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.737)).
	bottomSum8 setProperty: #objectName toValue: 'bottomSum8'.
	unnamedMorph7 addChild: bottomSum8.
	bottomSum9 := SudokuGivenNumberCell new.
	bottomSum9 changeTableLayout.
	bottomSum9 extent: (75@75).
	bottomSum9 position: (717@804).
	bottomSum9 hResizing: (#rigid).
	bottomSum9 vResizing: (#rigid).
	bottomSum9 minWidth: (2).
	bottomSum9 minHeight: (2).
	bottomSum9 cellInset: (0).
	bottomSum9 cellPositioning: (#center).
	bottomSum9 cellSpacing: (#none).
	bottomSum9 layoutInset: (0).
	bottomSum9 listCentering: (#center).
	bottomSum9 listDirection: (#topToBottom).
	bottomSum9 listSpacing: (#none).
	bottomSum9 maxCellSize: (1073741823).
	bottomSum9 minCellSize: (0).
	bottomSum9 wrapCentering: (#center).
	bottomSum9 wrapDirection: (#none).
	bottomSum9 frameFractions: (0@0 corner: 1@1).
	bottomSum9 frameOffsets: (0@0 corner: 0@0).
	bottomSum9 balloonText: (nil).
	bottomSum9 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.737)).
	bottomSum9 setProperty: #objectName toValue: 'bottomSum9'.
	unnamedMorph7 addChild: bottomSum9.
	unnamedMorph9 := SudokuGivenNumberCell new.
	unnamedMorph9 changeTableLayout.
	unnamedMorph9 extent: (85@75).
	unnamedMorph9 position: (794@804).
	unnamedMorph9 hResizing: (#rigid).
	unnamedMorph9 vResizing: (#rigid).
	unnamedMorph9 minWidth: (2).
	unnamedMorph9 minHeight: (2).
	unnamedMorph9 cellInset: (0).
	unnamedMorph9 cellPositioning: (#center).
	unnamedMorph9 cellSpacing: (#none).
	unnamedMorph9 layoutInset: (0).
	unnamedMorph9 listCentering: (#center).
	unnamedMorph9 listDirection: (#topToBottom).
	unnamedMorph9 listSpacing: (#none).
	unnamedMorph9 maxCellSize: (1073741823).
	unnamedMorph9 minCellSize: (0).
	unnamedMorph9 wrapCentering: (#center).
	unnamedMorph9 wrapDirection: (#none).
	unnamedMorph9 frameFractions: (0@0 corner: 1@1).
	unnamedMorph9 frameOffsets: (0@0 corner: 0@0).
	unnamedMorph9 balloonText: (nil).
	unnamedMorph9 balloonColor: ((TranslucentColor r: 0.92 g: 0.92 b: 0.706 alpha: 0.737)).
	unnamedMorph7 addChild: unnamedMorph9.
	panel addChild: unnamedMorph7.
	panel setProperty: #uiClassName toValue: 'SudokuGUIOutsideSum'.! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
leftSum1
	"No description given."
	
	^ leftSum1! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
leftSum2
	"No description given."
	
	^ leftSum2! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
leftSum3
	"No description given."
	
	^ leftSum3! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
leftSum4
	"No description given."
	
	^ leftSum4! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
leftSum5
	"No description given."
	
	^ leftSum5! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
leftSum6
	"No description given."
	
	^ leftSum6! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
leftSum7
	"No description given."
	
	^ leftSum7! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
leftSum8
	"No description given."
	
	^ leftSum8! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
leftSum9
	"No description given."
	
	^ leftSum9! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
rightSum1
	"No description given."
	
	^ rightSum1! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
rightSum2
	"No description given."
	
	^ rightSum2! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
rightSum3
	"No description given."
	
	^ rightSum3! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
rightSum4
	"No description given."
	
	^ rightSum4! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
rightSum5
	"No description given."
	
	^ rightSum5! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
rightSum6
	"No description given."
	
	^ rightSum6! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
rightSum7
	"No description given."
	
	^ rightSum7! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
rightSum8
	"No description given."
	
	^ rightSum8! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
rightSum9
	"No description given."
	
	^ rightSum9! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
topSum1
	"No description given."
	
	^ topSum1! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
topSum2
	"No description given."
	
	^ topSum2! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
topSum3
	"No description given."
	
	^ topSum3! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
topSum4
	"No description given."
	
	^ topSum4! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
topSum5
	"No description given."
	
	^ topSum5! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
topSum6
	"No description given."
	
	^ topSum6! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
topSum7
	"No description given."
	
	^ topSum7! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
topSum8
	"No description given."
	
	^ topSum8! !

!SudokuGUIOutsideSumUi methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:57'!
topSum9
	"No description given."
	
	^ topSum9! !

Morph subclass: #SudokuGUI
	instanceVariableNames: 'grid numbers hintButton takeStepButton playButton pauseButton buttonWrapper strategyDisplay layoutWrapper slider sudokuCellDict'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-GUI'!

!SudokuGUI methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:41'!
buttonWrapper

	^ buttonWrapper! !

!SudokuGUI methodsFor: 'accessing' stamp: 'mg 6/30/2014 13:28'!
cellAtX: xCoord Y: yCoord
	"This method returns the SudokuCell, that belongs to the given coordinates."
	^ sudokuCellDict at: xCoord@yCoord ifAbsent: [nil]! !

!SudokuGUI methodsFor: 'updating' stamp: 'mg 6/30/2014 13:28'!
changeColorOfDiagonalCellsTo: aColor

	(1 to: 9) do: 
		[:i |	(self cellAtX: i Y: i) color: aColor.
			(self cellAtX: i Y: 9-i+1) color: aColor].! !

!SudokuGUI methodsFor: 'initialization' stamp: 'MR 2/2/2013 17:16:13.196'!
connectSubmorphs 

	layoutWrapper addMorph: grid.
	layoutWrapper addMorph: buttonWrapper after: grid. 
	self addMorph: layoutWrapper.
	self addMorph: slider after: layoutWrapper.
	self addMorph: strategyDisplay after: slider.! !

!SudokuGUI methodsFor: 'initialization' stamp: 'MR 1/12/2013 16:53:39.154'!
createButtonWrapper

	buttonWrapper := RectangleMorph new. 
	buttonWrapper 	
		layoutPolicy: TableLayout new;
		listDirection: #leftToRight;
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		borderWidth: 3;
		borderColor: Color black;
		cellInset: 3;
		color: Color black.! !

!SudokuGUI methodsFor: 'initialization' stamp: 'MR 1/12/2013 23:50:14.102'!
createButtons: sudokuSolver
	"This method creates three buttons that are used to take the next step of solving the Sudoku."
	
	self 
		createButtonWrapper;
		createTakeStepButton: sudokuSolver;
		createPlayPauseButton: sudokuSolver.
		
	buttonWrapper 
		addMorph: takeStepButton;
		addMorph: playButton after: takeStepButton.
		
	layoutWrapper addMorph: buttonWrapper after: grid. 
	! !

!SudokuGUI methodsFor: 'initialization' stamp: 'mg 6/30/2014 17:24'!
createCellsFrom: aMorph
	"This method creates all sudoku cells. Each cell is like the given morph."
	| box |
	
	box := SudokuBuilder createBoxFromMorph: aMorph copy.
	box borderWidth: 2.
	^ (SudokuBuilder createBoxFromMorph: box)! !

!SudokuGUI methodsFor: 'initialization' stamp: 'mg 6/18/2014 14:39'!
createConstraintButtons
	"This method creates a hint button."
	
	self 
		createButtonWrapper;
		createHintButton.
		
	buttonWrapper 
		addMorph: hintButton.
		
	layoutWrapper addMorph: buttonWrapper after: grid. 
	! !

!SudokuGUI methodsFor: 'initialization' stamp: 'mg 6/18/2014 14:42'!
createHintButton
	"This method creates a help button. If the use clicks on it, a random number (of a valid solution) will be given."
	hintButton := SimpleButtonMorph new.
	hintButton 
		actionSelector: #giveHint;
		target: self;
		label: 'Give a hint' font: (((TextStyle named: 'BitstreamVeraSans') 
							    				fontOfSize: 36) emphasized: 1);
		layoutPolicy: TableLayout new;
		color: Color white;
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		layoutInset: 10;
		wrapCentering: #center.
	
	! !

!SudokuGUI methodsFor: 'initialization' stamp: 'MR 1/12/2013 17:41:41.884'!
createPlayPauseButton: sudokuSolver
	"This method creates a button that is used to take steps of solving the Sudoku until pause is pressed."
	
	playButton := SudokuPlayButton new.
	playButton  
		sudokuSolver: sudokuSolver;
		actionSelector: #playOrPause;
		target: playButton;
		layoutPolicy: TableLayout new;
		color: Color white;
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		layoutInset: 10;
		wrapCentering: #center.
		
	playButton 
		playForm: (Form fromFileNamed: 'Play.png');
		pauseForm: (Form fromFileNamed: 'Pause.png').! !

!SudokuGUI methodsFor: 'initialization' stamp: 'MR 2/2/2013 19:15:56.552'!
createSlider: aSudokuSolver

	slider := SudokuSlider new.
	slider sudokuSolver: aSudokuSolver.
	^ slider.! !

!SudokuGUI methodsFor: 'initialization' stamp: 'MR 2/2/2013 17:15:51.111'!
createStrategyDisplay: anOrderedCollection

	strategyDisplay := SudokuStrategyDisplay new.
	strategyDisplay initializeStrategyTexts: anOrderedCollection.
	strategyDisplay highlightLine: 1.
	^strategyDisplay! !

!SudokuGUI methodsFor: 'initialization' stamp: 'MR 1/12/2013 20:10:12.713'!
createTakeStepButton: sudokuSolver
	"This method creates a button that is used to take the next step of solving the Sudoku."
	takeStepButton := SimpleButtonMorph new.
	takeStepButton 
		actionSelector: #takeStep;
		target: sudokuSolver;
		label: 'Take Step' font: (((TextStyle named: 'BitstreamVeraSans') 
							    				fontOfSize: 36) emphasized: 1);
		layoutPolicy: TableLayout new;
		color: Color white;
		vResizing: #spaceFill;
		width: 69*9;
		layoutInset: 10;
		wrapCentering: #center.! !

!SudokuGUI methodsFor: 'solving' stamp: 'mg 6/23/2014 14:29'!
giveHint
	"This method sets one empty cell to a correct number and adds an equality constraint for it on the sudoku grid."
	| randomX randomY randomGenerator number textMorph |
	(sudokuCellDict values anySatisfy: [:cell | cell submorphs first asText string = ''])
		ifFalse: [self inform: 'There are no empty cells.'. ^ self].
	randomX := 1.
	randomY := 1.
	randomGenerator := Random new.
	textMorph := (sudokuCellDict at: randomX@randomY) submorphs first.
	[textMorph asText string = ''] 
		whileFalse: [randomX := randomGenerator nextInt: 9.
					randomY := randomGenerator nextInt: 9.
					textMorph := (sudokuCellDict at: randomX@randomY) submorphs first.].
	number := self numbers at: randomX at: randomY.
	self numbers createConstraintAt: randomX at: randomY with: number. "should not fail"
	textMorph updateColor: Color red.
	textMorph updateTextWith: number.! !

!SudokuGUI methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:35'!
grid

	^ grid! !

!SudokuGUI methodsFor: 'initialization' stamp: 'mg 6/18/2014 14:02'!
initialize

	super initialize.
	
	self 
		initializeThisMorph;
		initializeGrid;
		initializeLayoutWrapper.
		
	layoutWrapper addMorph: grid.
	self addMorph: layoutWrapper.

	sudokuCellDict := Dictionary new.! !

!SudokuGUI methodsFor: 'initialization' stamp: 'mg 6/30/2014 17:22'!
initializeGrid 

	grid := self createCellsFrom: SudokuCandidateCell new.
	grid borderWidth: 0.! !

!SudokuGUI methodsFor: 'initialization' stamp: 'MR 1/12/2013 16:00:26.561'!
initializeLayoutWrapper

	layoutWrapper := Morph new. 
	layoutWrapper 	
		layoutPolicy: TableLayout new;
		listDirection: #topToBottom;
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		layoutInset: 0;
		cellInset: 0;
		borderWidth: 0;
		color: Color orange.! !

!SudokuGUI methodsFor: 'initialization' stamp: 'MR 1/12/2013 16:28:49.124'!
initializeThisMorph

	self
		layoutPolicy: TableLayout new;
		listDirection: #leftToRight;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap;
		borderWidth: 5;
		cellInset: 0;
		borderColor: Color black;
		color: Color black.! !

!SudokuGUI methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:41'!
layoutWrapper

	^ layoutWrapper! !

!SudokuGUI methodsFor: 'accessing' stamp: 'mg 6/18/2014 14:44'!
numbers

	^ numbers! !

!SudokuGUI methodsFor: 'accessing' stamp: 'mg 6/18/2014 14:44'!
numbers: aSudokuGrid

	numbers := aSudokuGrid.! !

!SudokuGUI methodsFor: 'accessing' stamp: 'MR 12/23/2012 18:09'!
setButtonName: aString

	takeStepButton submorphs first  contents: aString! !

!SudokuGUI methodsFor: 'updating' stamp: 'MR 2/2/2013 18:34:47.088'!
setCells: aMatrix ofType: aSymbol
	"Sets the values of the grid according to the given matrix. The values can be #number or #candidates."
	| a b c d |
	
	a := 0.
	grid submorphs do:
	[:threeBoxes | a := a + 1. b := 0. threeBoxes submorphs do:
		[:box | b := b + 1. c := 0. box submorphs do:
			[:threeCells | c := c + 1. d := 0. threeCells submorphs do:
				[:cell | d := d + 1. cell 
									updateContent: (aMatrix at: ((b-1)*3+d) at: ((a-1)*3+c)) 
									ofType: aSymbol]]]]. ! !

!SudokuGUI methodsFor: 'updating' stamp: 'mg 6/23/2014 14:30'!
setMatrixCoordinatesOfSudoku: aSudokuGrid
	"Sets the x and y coordinates of the corresponding matrix."
	| a b c d |
	
	a := 0.
	grid submorphs do:
	[:threeBoxes | a := a + 1. b := 0. threeBoxes submorphs do:
		[:box | b := b + 1. c := 0. box submorphs do:
			[:threeCells | c := c + 1. d := 0. threeCells submorphs do:
				[:cell | d := d + 1. 
						cell sudokuModel: aSudokuGrid; xCoord: ((b-1)*3+d); yCoord: ((a-1)*3+c).
						sudokuCellDict at: ((b-1)*3+d)@((a-1)*3+c) put: cell.]]]]. ! !

!SudokuGUI methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:39'!
sudokuCellDict

	^ sudokuCellDict! !

!SudokuGUI methodsFor: 'initialization' stamp: 'mg 6/25/2014 13:32'!
superInitialize

	super initialize.! !

!SudokuGUI methodsFor: 'updating' stamp: 'MG 12/7/2012 15:18'!
updateCells: aMatrix ofType: aSymbol
	"Sets the values of the grid according to the given matrix. The values can be #number or #candidates."
	| a b c d |
	
	a := 0.
	grid submorphs do:
	[:threeBoxes | a := a + 1. b := 0. threeBoxes submorphs do:
		[:box | b := b + 1. c := 0. box submorphs do:
			[:threeCells | c := c + 1. d := 0. threeCells submorphs do:
				[:cell | d := d + 1. cell 
									updateContent: (aMatrix at: ((b-1)*3+d) at: ((a-1)*3+c)) 
									ofType: aSymbol]]]]. ! !

SudokuGUI subclass: #SudokuGUIOutsideSum
	instanceVariableNames: 'ui'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-GUI'!

!SudokuGUIOutsideSum methodsFor: 'initialization' stamp: 'mg 6/25/2014 13:42'!
initialize

	super superInitialize.
 	self ui setupUi: self.
	grid := self ui grid grid.
	sudokuCellDict := self ui grid sudokuCellDict.
	numbers := self ui grid numbers.
	layoutWrapper := self ui grid layoutWrapper.
	buttonWrapper := self ui grid buttonWrapper.! !

!SudokuGUIOutsideSum methodsFor: 'updating' stamp: 'mg 6/25/2014 13:58'!
setOutsideSums

	| topSums bottomSums leftSums rightSums |
	topSums := self numbers topSums.
	self ui topSum1 setNumericValue: (topSums at: 1).
	self ui topSum2 setNumericValue: (topSums at: 2).
	self ui topSum3 setNumericValue: (topSums at: 3).
	self ui topSum4 setNumericValue: (topSums at: 4).
	self ui topSum5 setNumericValue: (topSums at: 5).
	self ui topSum6 setNumericValue: (topSums at: 6).
	self ui topSum7 setNumericValue: (topSums at: 7).
	self ui topSum8 setNumericValue: (topSums at: 8).
	self ui topSum9 setNumericValue: (topSums at: 9).

	bottomSums := self numbers bottomSums.
	self ui bottomSum1 setNumericValue: (bottomSums at: 1).
	self ui bottomSum2 setNumericValue: (bottomSums at: 2).
	self ui bottomSum3 setNumericValue: (bottomSums at: 3).
	self ui bottomSum4 setNumericValue: (bottomSums at: 4).
	self ui bottomSum5 setNumericValue: (bottomSums at: 5).
	self ui bottomSum6 setNumericValue: (bottomSums at: 6).
	self ui bottomSum7 setNumericValue: (bottomSums at: 7).
	self ui bottomSum8 setNumericValue: (bottomSums at: 8).
	self ui bottomSum9 setNumericValue: (bottomSums at: 9).

	leftSums := self numbers leftSums.
	self ui leftSum1 setNumericValue: (leftSums at: 1).
	self ui leftSum2 setNumericValue: (leftSums at: 2).
	self ui leftSum3 setNumericValue: (leftSums at: 3).
	self ui leftSum4 setNumericValue: (leftSums at: 4).
	self ui leftSum5 setNumericValue: (leftSums at: 5).
	self ui leftSum6 setNumericValue: (leftSums at: 6).
	self ui leftSum7 setNumericValue: (leftSums at: 7).
	self ui leftSum8 setNumericValue: (leftSums at: 8).
	self ui leftSum9 setNumericValue: (leftSums at: 9).

	rightSums := self numbers rightSums.
	self ui rightSum1 setNumericValue: (rightSums at: 1).
	self ui rightSum2 setNumericValue: (rightSums at: 2).
	self ui rightSum3 setNumericValue: (rightSums at: 3).
	self ui rightSum4 setNumericValue: (rightSums at: 4).
	self ui rightSum5 setNumericValue: (rightSums at: 5).
	self ui rightSum6 setNumericValue: (rightSums at: 6).
	self ui rightSum7 setNumericValue: (rightSums at: 7).
	self ui rightSum8 setNumericValue: (rightSums at: 8).
	self ui rightSum9 setNumericValue: (rightSums at: 9).
! !

!SudokuGUIOutsideSum methodsFor: 'accessing' stamp: 'mg 6/25/2014 13:03'!
ui

	^ ui ifNil: [ui := SudokuGUIOutsideSumUi new]! !

TextMorph subclass: #SudokuTextMorph
	instanceVariableNames: 'highlightColor removeCandidatesColor updateColor normalColor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-GUI'!

!SudokuTextMorph methodsFor: 'event handling' stamp: 'mg 6/18/2014 12:50'!
handlesKeyboard: evt

	^ true! !

!SudokuTextMorph methodsFor: 'event handling' stamp: 'mg 6/18/2014 12:50'!
handlesMouseOver: evt

	^ true! !

!SudokuTextMorph methodsFor: 'accessing' stamp: 'MG 12/15/2012 14:52'!
highlightColor
	"Answer the value of highlightColor"

	^ highlightColor! !

!SudokuTextMorph methodsFor: 'accessing' stamp: 'MG 12/15/2012 14:52'!
highlightColor: aColor
	"Set the value of highlightColor"

	highlightColor := aColor! !

!SudokuTextMorph methodsFor: 'initialization' stamp: 'mg 6/19/2014 11:19'!
initialize 

	super initialize.
	self 
		updateColor: Color blue;
		normalColor: Color black;
		highlightColor: Color green.! !

!SudokuTextMorph methodsFor: 'event handling' stamp: 'mg 6/23/2014 14:38'!
keyStroke: anEvent
	"This method handles a keystroke event. 
	If the user input is a number between 1 and 9, the new number is suggested to the sudoku model. If the sudoku is satisfiable, the cell text is updated.
	If the user input is an arrow key, the keyboard focus is moved to the next cell in the given direction.
	If the user input is <delete> or <backspace>, the current number will be deleted."
	| numberFromAsciiInput |
	numberFromAsciiInput := anEvent keyValue - 48.
	(numberFromAsciiInput between: 1 and: 9) 
		ifTrue: ["try solving, set number if possible" 
				[self owner suggestNewValueToSudokuModel: numberFromAsciiInput] 
					ifError: [self inform: 'The input value ', numberFromAsciiInput, ' made the Sudoku unsatisfiable.'. 
							self keyboardFocusChange: false.
							^ self].
				self updateTextWith: numberFromAsciiInput.
				^ self].

	"navigate on arrow keys"
	(anEvent keyCharacter = Character arrowDown)
		ifTrue: [self owner moveFocusBy: 1@0 event: anEvent. ^ self].
	(anEvent keyCharacter = Character arrowUp)
		ifTrue: [self owner moveFocusBy: -1@0 event: anEvent. ^ self].
	(anEvent keyCharacter = Character arrowLeft)
		ifTrue: [self owner moveFocusBy: 0@(-1) event: anEvent. ^ self].
	(anEvent keyCharacter = Character arrowRight)
		ifTrue: [self owner moveFocusBy: 0@1 event: anEvent. ^ self].

	(anEvent keyCharacter = Character backspace
		or: [anEvent keyCharacter = Character delete])
			ifTrue: ["delete number in SudokuGrid"
					(self text = '') ifTrue: [^ self].
					self owner deleteNumberInSudokuModel.
					self updateColor: Color blue.
					^ self updateTextWith: ''].

	self inform: 'Invalid input for a Sudoku: ', anEvent keyString, '. Please enter a number between 1 and 9.'.
	self keyboardFocusChange: false.
	! !

!SudokuTextMorph methodsFor: 'event handling' stamp: 'mg 6/18/2014 13:38'!
mouseEnter: evt

	super mouseEnter: evt.
	evt hand newKeyboardFocus: self.
	self
		handleInteraction: [editor mouseDown: evt]
		fromEvent: evt.! !

!SudokuTextMorph methodsFor: 'accessing' stamp: 'MR 12/9/2012 23:58'!
normalColor
	"Answer the value of normalColor"

	^ normalColor! !

!SudokuTextMorph methodsFor: 'accessing' stamp: 'MR 12/9/2012 23:59'!
normalColor: aColor
	"Set the value of normalColor"

	normalColor := aColor! !

!SudokuTextMorph methodsFor: 'accessing' stamp: 'MG 12/15/2012 16:45'!
removeCandidatesColor
	"Answer the value of removeCandidatesColor"

	^ removeCandidatesColor! !

!SudokuTextMorph methodsFor: 'accessing' stamp: 'MG 12/15/2012 16:45'!
removeCandidatesColor: anObject
	"Set the value of removeCandidatesColor"

	removeCandidatesColor := anObject! !

!SudokuTextMorph methodsFor: 'visual properties' stamp: 'MR 2/1/2013 23:22:19.621'!
setToHighlightColor

	self backgroundColor: highlightColor; textColor: Color white.! !

!SudokuTextMorph methodsFor: 'visual properties' stamp: 'MR 12/9/2012 23:58'!
setToNormalColor

	self textColor: normalColor! !

!SudokuTextMorph methodsFor: 'visual properties' stamp: 'MR 2/1/2013 12:07:00.193'!
setToRemoveCandidatesColor

	self backgroundColor: removeCandidatesColor; textColor: Color white.! !

!SudokuTextMorph methodsFor: 'visual properties' stamp: 'MR 12/9/2012 23:57'!
setToUpdateColor

	self textColor: updateColor! !

!SudokuTextMorph methodsFor: 'visual properties' stamp: 'MR 2/1/2013 23:25:18.651'!
setToWhiteBackground
	self backgroundColor: Color white; textColor: normalColor.  ! !

!SudokuTextMorph methodsFor: 'accessing' stamp: 'MR 12/9/2012 23:58'!
updateColor
	"Answer the value of updateColor"

	^ updateColor! !

!SudokuTextMorph methodsFor: 'accessing' stamp: 'MR 12/9/2012 23:59'!
updateColor: aColor
	"Set the value of updateColor"

	updateColor := aColor! !

!SudokuTextMorph methodsFor: 'event handling' stamp: 'MG 12/14/2012 15:48'!
updateTextWith: aValue

	(aValue = self text asString)
		ifTrue: [self setToNormalColor]
		ifFalse: [self 
					setNumericValue: aValue;
					setToUpdateColor].! !

Slider subclass: #SudokuSlider
	instanceVariableNames: 'sudokuSolver maxRelativeValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-GUI'!

!SudokuSlider methodsFor: 'initialization' stamp: 'MR 2/2/2013 19:06:22.71'!
initialize 

	super initialize.
	
	maxRelativeValue := 0.
	
	self 
		color: Color white;
		sliderColor: Color gray;
		value: 1;
		height: 758;
		width: 20.! !

!SudokuSlider methodsFor: 'accessing' stamp: 'MR 2/2/2013 19:10:08.506'!
maxRelativeValue
	"Answer the value of maxRelativeValue"

	^ maxRelativeValue! !

!SudokuSlider methodsFor: 'accessing' stamp: 'MR 2/2/2013 19:10:23.408'!
maxRelativeValue: anInteger
	"Set the value of maxRelativeValue"

	maxRelativeValue := anInteger! !

!SudokuSlider methodsFor: 'other events' stamp: 'MR 2/2/2013 19:15:03.186'!
mouseUpInSlider: anEvent

	| indexOfHistory indexAsFloatStartingWith0|
	super mouseUpInSlider: anEvent.
	
	indexAsFloatStartingWith0 := value * (maxRelativeValue - 1).
	indexOfHistory := indexAsFloatStartingWith0 // 1 + 1.
	sudokuSolver takeStepFromHistoryAt: (indexOfHistory).
	self relativeValue: indexOfHistory! !

!SudokuSlider methodsFor: 'accessing' stamp: 'MR 2/2/2013 19:26:25.241'!
relativeValue: anInteger
	| stepWidth |
	stepWidth := 1.0 / (maxRelativeValue - 1) asFloat .
	self value: anInteger - 1 * stepWidth! !

!SudokuSlider methodsFor: 'accessing' stamp: 'MR 2/2/2013 19:39:43.24'!
sliderThickness
	^100! !

!SudokuSlider methodsFor: 'accessing' stamp: 'MR 2/2/2013 19:05:18.499'!
sudokuSolver: anObject
	"Set the value of sudokuSolver"

	sudokuSolver := anObject! !

Matrix subclass: #SudokuGrid
	instanceVariableNames: 'sudokuGui solver equalityConstraints'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Core'!

SudokuGrid subclass: #SudokuDiagonal
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Core'!

!SudokuDiagonal class methodsFor: 'instance creation' stamp: 'mg 6/23/2014 16:45'!
newConstrainedSudokuFromFile: aFileName

	| sudoku |
	sudoku := super newConstrainedSudokuFromFile: aFileName.
	sudoku sudokuGui changeColorOfDiagonalCellsTo: Color yellow.
	^ sudoku! !

!SudokuDiagonal class methodsFor: 'instance creation' stamp: 'mg 6/23/2014 16:18'!
newFromFile: aFileName

	^ super newFromFile: aFileName! !

!SudokuDiagonal methodsFor: 'solving' stamp: 'mg 7/4/2014 14:20'!
createConstraints
	"This method creates all necessary constraints to solve a Diagonal Sudoku puzzle with Z3."
	solver variableType: 'Int'.
	solver autoSolve: false.

	Transcript show: 'milliseconds to create stay constraints: ', [
	self addConstraintsForAllGivenNumbers]
	timeToRun; cr.

	Transcript show: 'create domain constraints: ', [
	[self allSatisfy: [:val | val between: 1 and: 9]] alwaysSolveWith: solver]
	timeToRun; cr.

	Transcript show: 'milliseconds to create row, column and box constraints: ', [
	(1 to: self rowCount) do: [:index |
		[(self atRow: index) allDifferent &
		(self atColumn: index) allDifferent &
		(self atBox: index) allDifferent] alwaysSolveWith: solver]]
	timeToRun; cr.

	Transcript show: 'milliseconds to create diagonal constraints: ', [
	[self diagonalFromTopLeftToBottomRight allDifferent] alwaysSolveWith: solver.
	[self diagonalFromTopRightToBottomLeft allDifferent] alwaysSolveWith: solver]
	timeToRun; cr..

	Transcript show: 'milliseconds to solve the sudoku constraints: ', [solver solve] timeToRun; cr.! !

!SudokuDiagonal methodsFor: 'accessing' stamp: 'mg 6/23/2014 16:16'!
diagonalFromTopLeftToBottomRight
	"This method returns a new collection of all cells in the diagonal."
	| diagonal |
	diagonal := Array new: nrows.
	(1 to: nrows)
		do: [:i | diagonal at: i put: (self at: i at: i)].
	^ diagonal! !

!SudokuDiagonal methodsFor: 'accessing' stamp: 'mg 6/23/2014 16:25'!
diagonalFromTopRightToBottomLeft
	"This method returns a new collection of all cells in the diagonal."
	| diagonal |
	diagonal := Array new: nrows.
	(1 to: nrows)
		do: [:i | diagonal at: i put: (self at: i at: nrows-i+1)].
	^ diagonal! !

!SudokuGrid class methodsFor: 'accessing' stamp: 'mg 5/28/2014 12:59'!
boxIndexDict
"This method returns a dictionary that associate the index of a 3x3 box with the position of the top left cell of the box."
	^ Dictionary new
		at: 1 put: 1@1;
		at: 2 put: 1@4;
		at: 3 put: 1@7;
		at: 4 put: 4@1;
		at: 5 put: 4@4;
		at: 6 put: 4@7;
		at: 7 put: 7@1;
		at: 8 put: 7@4;
		at: 9 put: 7@7;
		yourself! !

!SudokuGrid class methodsFor: 'accessing' stamp: 'mg 2/2/2013 17:05:27.422'!
boxIndexFromCellIndex: aPoint
	^ 
	(aPoint x - 1 \\ 3) 	"1,4,7 -> 0     2,5,8 -> 1    3,6,9 -> 2"
	* 3
	+ (aPoint y - 1 \\ 3)
	+ 1
	! !

!SudokuGrid class methodsFor: 'accessing' stamp: 'mg 2/2/2013 16:13:35.687'!
boxNumberFromIndex: aPoint
	| xBox yBox |
	xBox := aPoint x - 1 // 3.
	yBox := aPoint y - 1 // 3.
	^ xBox * 3 + yBox + 1! !

!SudokuGrid class methodsFor: 'instance creation' stamp: 'mg 5/28/2014 13:14'!
getCandidatesMatrixFrom: aMatrix

	| allCandidates candidates |
	allCandidates := OrderedCollection new.
	1 to: 9 do:
		[:i | allCandidates add: i].
	
	candidates := Matrix new: 9.
	1 to: 9 do:
		[:i | 1 to: 9 do:
			[:j | ((aMatrix at: i at: j) = 0)
					ifTrue: [candidates at: i at: j put: allCandidates copy]
					ifFalse: [candidates at: i at: j put: (OrderedCollection with: (aMatrix at: i at: j))]]].

	^ candidates! !

!SudokuGrid class methodsFor: 'accessing' stamp: 'mg 1/10/2013 22:47:16.215'!
indexOfbox: boxIndex cell: cellIndex
	^ ((self topLeftCellOfBox: boxIndex) x + (cellIndex-1 // 3))
		@ ((self topLeftCellOfBox: boxIndex) y + (cellIndex-1 \\ 3))! !

!SudokuGrid class methodsFor: 'accessing' stamp: 'mg 2/2/2013 23:39:40.476'!
missingLineNumbersInBoxForLine: lineNumber
	| dict |
	dict := Dictionary new.
	dict 
		add: 1 -> (OrderedCollection with: 2 with: 3);
		add: 2 -> (OrderedCollection with: 1 with: 3);
		add: 3 -> (OrderedCollection with: 1 with: 2);
		add: 4 -> (OrderedCollection with: 5 with: 6);
		add: 5 -> (OrderedCollection with: 4 with: 6);
		add: 6 -> (OrderedCollection with: 4 with: 5);
		add: 7 -> (OrderedCollection with: 8 with: 9);
		add: 8 -> (OrderedCollection with: 7 with: 9);
		add: 9 -> (OrderedCollection with: 7 with: 8).
	^ dict at: lineNumber! !

!SudokuGrid class methodsFor: 'instance creation' stamp: 'mg 7/4/2014 14:39'!
newConstrainedSudokuFromFile: aFileName

	| sudoku |
	sudoku := self newFromFile: aFileName.
	sudoku createConstraints.
	sudoku sudokuGui openInWorld.
	^ sudoku! !

!SudokuGrid class methodsFor: 'instance creation' stamp: 'mg 6/10/2014 15:46'!
newFrom: aMatrix

	^ self rows: aMatrix rowCount columns: aMatrix columnCount contents: aMatrix asArray deepCopy! !

!SudokuGrid class methodsFor: 'instance creation' stamp: 'mg 6/24/2014 15:51'!
newFromFile: aFileName
	| file numberList numbers numbersMatrix |

	numberList := OrderedCollection new.
	file := MultiByteFileStream fileNamed: aFileName.
	file upToEnd asString substrings
		collect: [:row | row do: [:digit | numberList add: digit digitValue]].
	numbers := Matrix new: 9 tabulate: [:i :j | numberList at: (j \\ 10 + (i-1*9))].
	file close.
	
	numbersMatrix := self newFrom: numbers.

	numbersMatrix sudokuGui: SudokuGUI new.
	numbersMatrix sudokuGui updateCells: numbersMatrix ofType: #number.
	numbersMatrix sudokuGui setMatrixCoordinatesOfSudoku: numbersMatrix.
	numbersMatrix sudokuGui numbers: numbersMatrix.
	numbersMatrix sudokuGui createConstraintButtons.

	^ numbersMatrix! !

!SudokuGrid class methodsFor: 'accessing' stamp: 'mg 5/28/2014 12:59'!
topLeftCellOfBox: index

	^ self boxIndexDict at: index! !

!SudokuGrid methodsFor: 'solving' stamp: 'mg 6/23/2014 14:24'!
addConstraintsForAllGivenNumbers
	"This method creates equality constraints for all given numbers (e.g. '(self at: 1 at: 5) = 3'), so that the given numbers cannot be changed by the solver. Additionally, these constraints have to be stored, so deleting a number will disable the corresponding constraint."
	self withIndicesDo: [:val :row :column | 
		val ~= 0 ifTrue: [equalityConstraints at: row at: column put: 
							([(self at: row at: column) = val] alwaysSolveWith: solver)]].! !

!SudokuGrid methodsFor: 'accessing boxes' stamp: 'mg 6/23/2014 14:24'!
atBox: index
	"This method returns a new collection of all cells in the box at index."
	| box topLeftIndex |
	(index between: 1 and: nrows)
		ifFalse: [self error: 'box subscript out of range'].
	box := Array new: nrows.
	topLeftIndex := self class topLeftCellOfBox: index.
	0 to: 2 do:
		[:x | 0 to: 2 do:
			[:y | box at: (x*3 + y + 1) put: (self at: topLeftIndex x + x at: topLeftIndex y + y)]].
	^ box! !

!SudokuGrid methodsFor: 'accessing boxes' stamp: 'mg 1/10/2013 22:39:32.542'!
boxCount
	^ self rowCount! !

!SudokuGrid methodsFor: 'solving' stamp: 'mg 6/23/2014 14:27'!
createConstraintAt: x at: y with: aValue
	"This method is called for new values. It creates the equality constraint for the new number and stores it. The method throws an error, if the constraints are unsatisfiable. An old value (and constraint) for the same cell will be overriden."
	| constraint |
	constraint := [(self at: x at: y) = aValue] alwaysSolveWith: solver.
	((self at: x at: y) = aValue)
		ifFalse: [self disableEqualityConstraintAt: x at: y.
				[solver solve] ifError: [constraint disable. ConstraintUnsatisfiableError new signal]].
	equalityConstraints at: x at: y put: constraint.! !

!SudokuGrid methodsFor: 'solving' stamp: 'mg 7/4/2014 14:10'!
createConstraints
	"This method creates all necessary constraints to solve a standard Sudoku puzzle with Z3."
	solver variableType: 'Int'.
	solver autoSolve: false.

	Transcript show: 'milliseconds to create stay constraints: ', [
	self addConstraintsForAllGivenNumbers]
	timeToRun; cr.

	Transcript show: 'create domain constraints: ', [
	[self allSatisfy: [:val | val between: 1 and: 9]] alwaysSolveWith: solver]
	timeToRun; cr.

	Transcript show: 'milliseconds to create row, column and box constraints: ', [
	(1 to: self rowCount) do: [:index |
		[(self atRow: index) allDifferent &
		(self atColumn: index) allDifferent &
		(self atBox: index) allDifferent] alwaysSolveWith: solver]]
	timeToRun; cr.

	Transcript show: 'milliseconds to solve the sudoku constraints: ', [solver solve] timeToRun; cr.! !

!SudokuGrid methodsFor: 'solving' stamp: 'mg 6/23/2014 14:27'!
disableEqualityConstraintAt: x at: y
	"This method is called, if a number is deleted. It disables the corresponding equality constraint and removes it."
	(equalityConstraints at: x at: y) ifNil: [Transcript show: 
		'WARNING - Cannot disable the equality constraint at (', x, ',', y, '), because there is no constraint to disable.'; cr.
		^ self].
	(equalityConstraints at: x at: y) disable.
	equalityConstraints at: x at: y put: nil.! !

!SudokuGrid methodsFor: 'initialize-release' stamp: 'mg 6/23/2014 13:51'!
initialize

	super initialize.
	solver := ConstraintSolver newZ3Solver.
	equalityConstraints := Matrix new: 9.! !

!SudokuGrid methodsFor: 'accessing' stamp: 'mg 6/10/2014 13:36'!
sudokuGui

	^ sudokuGui! !

!SudokuGrid methodsFor: 'accessing' stamp: 'mg 6/10/2014 13:36'!
sudokuGui: anObject

	sudokuGui := anObject! !

SudokuGrid subclass: #SudokuOutsideSum
	instanceVariableNames: 'topSums bottomSums leftSums rightSums'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Core'!

!SudokuOutsideSum class methodsFor: 'instance creation' stamp: 'mg 6/25/2014 13:45'!
newFromFile: aFileName
	"The given file should include the sums of the four borders. The order is top, bottom, left, right. The sums are read from left to right or top to bottom. All Sudoku cells are assumed to be empty."
	| sudoku file lines |
	sudoku := SudokuOutsideSum new: 9 element: 0.
	file := MultiByteFileStream fileNamed: aFileName.
	lines := file upToEnd lines.
	lines first substrings do: [:digit | sudoku topSums add: digit asNumber].
	lines second substrings do: [:digit | sudoku bottomSums add: digit asNumber].
	lines third substrings do: [:digit | sudoku leftSums add: digit asNumber].
	lines fourth substrings do: [:digit | sudoku rightSums add: digit asNumber].
	file close.

	sudoku sudokuGui: SudokuGUIOutsideSum new.
	sudoku sudokuGui updateCells: sudoku ofType: #number.
	sudoku sudokuGui setMatrixCoordinatesOfSudoku: sudoku.
	sudoku sudokuGui numbers: sudoku.
	sudoku sudokuGui createConstraintButtons.
	sudoku sudokuGui setOutsideSums.

	^ sudoku! !

!SudokuOutsideSum methodsFor: 'accessing' stamp: 'mg 6/24/2014 15:31'!
bottomSums

	^ bottomSums! !

!SudokuOutsideSum methodsFor: 'solving' stamp: 'mg 7/4/2014 14:28'!
createConstraints
	"This method creates all necessary constraints to solve an Outside Sum Sudoku puzzle with Z3."
	solver variableType: 'Int'.
	solver autoSolve: false.

	Transcript show: 'milliseconds to create sum constraints: ', [
	self createSumConstraints.]
	timeToRun; cr.

	Transcript show: 'create domain constraints: ', [
	[self allSatisfy: [:val | val between: 1 and: 9]] alwaysSolveWith: solver]
	timeToRun; cr.

	Transcript show: 'milliseconds to create row, column and box constraints: ', [
	(1 to: self rowCount) do: [:index |
		[(self atRow: index) allDifferent &
		(self atColumn: index) allDifferent &
		(self atBox: index) allDifferent] alwaysSolveWith: solver]]
	timeToRun; cr.

	Transcript show: 'milliseconds to solve the sudoku constraints: ', [solver solve] timeToRun; cr.! !

!SudokuOutsideSum methodsFor: 'solving' stamp: 'mg 6/24/2014 16:30'!
createSumConstraints
	"This method creates all sum constraints of an OusideSumSudoku. Each field outside the border defines the sum of the first three digits from that direction."

	(1 to: 9) do: [:i |
		[(self firstThreeCellsFrom: 1@(i) direction: 1@0) sum = (self topSums at: i) value] 
			alwaysSolveWith: solver.
		[(self firstThreeCellsFrom: 9@(i) direction: -1@0) sum = (self bottomSums at: i) value] 
			alwaysSolveWith: solver.
		[(self firstThreeCellsFrom: (i)@1 direction: 0@1) sum = (self leftSums at: i) value] 
			alwaysSolveWith: solver.
		[(self firstThreeCellsFrom: (i)@9 direction: 0@(-1)) sum = (self rightSums at: i) value] 
			alwaysSolveWith: solver.].! !

!SudokuOutsideSum methodsFor: 'accessing' stamp: 'mg 6/24/2014 16:11'!
firstThreeCellsFrom: aStartingPoint direction: aDirectionPoint
	"This method returns the three cells from aStartingPoint belong the given direction. 
	E.g., self firstThreeCellsFrom: 1@1 direction: 1@0 returns the cells at 1@1, 2@1 and 3@1,
		self firstThreeCellsFrom: 4@9 direction: 0@-1 returns the cells at 4@9, 4@8 and 4@7."
	| cells |
	cells := Array new: 3.
	(0 to: 2) do: [:i | cells at: i+1 put: 
		(self at: aStartingPoint x + (i * aDirectionPoint x) 
			  at: aStartingPoint y + (i * aDirectionPoint y))].
	^ cells! !

!SudokuOutsideSum methodsFor: 'initialize-release' stamp: 'mg 6/24/2014 15:35'!
initialize

	super initialize.
	topSums := OrderedCollection new.
	bottomSums := OrderedCollection new.
	leftSums := OrderedCollection new.
	rightSums := OrderedCollection new.! !

!SudokuOutsideSum methodsFor: 'accessing' stamp: 'mg 6/24/2014 15:31'!
leftSums

	^ leftSums! !

!SudokuOutsideSum methodsFor: 'accessing' stamp: 'mg 6/24/2014 15:31'!
rightSums

	^ rightSums! !

!SudokuOutsideSum methodsFor: 'accessing' stamp: 'mg 6/24/2014 15:31'!
topSums

	^ topSums! !
